#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['pyomo2lp', 'pyomo2nl', 'pyomo2osil']


import argparse
from coopr.opt import ProblemFormat
from coopr.opt.base import SolverFactory
from pyutilib.misc import Options, Container

import util
import pyomo

def create_parser(cmd):
    #
    #
    # Setup command-line options
    #
    #
    parser = argparse.ArgumentParser(
                usage = '%s [options] <model_file> [<data_files>]' % cmd
                )
    pyomo.add_model_group(parser)
    pyomo.add_logging_group(parser)
    pyomo.add_misc_group(parser)
    parser.add_argument('model_file', action='store', nargs='?', default='', help='A Python module that defines a Pyomo model')
    parser.add_argument('data_files', action='store', nargs='*', default=[], help='Pyomo data files that defined data used to create a model instance')
    return parser

format = None

def convert(options=Options(), parser=None):
    global format
    if options.save_model is None:
        if format == ProblemFormat.cpxlp:
            options.save_model = 'unknown.lp'
        else:
            options.save_model = 'unknown.'+str(format)
    options.format = format
    #
    data = Options(options=options)
    #
    if options.help_components:
        util.print_components(data)
        return Container()
    #
    util.setup_environment(data)
    #
    util.apply_preprocessing(data, parser=parser)
    if data.error:
        return Container()
    #
    model_data = util.create_model(data)
    #
    util.finalize(data, model=model_data.model)
    #
    model_data.options = options
    return model_data

def pyomo2lp(args=None):
    global format
    parser = create_parser('pyomo2lp')
    format = ProblemFormat.cpxlp
    return util.run_command(command=convert, parser=parser, args=args, name='pyomo2lp').retval

def pyomo2lp_main(args=None):
    ans = pyomo2lp(args)
    if ans is None:
        return 0
    return 1

def pyomo2nl(args=None):
    global format
    parser = create_parser('pyomo2nl')
    format = ProblemFormat.nl
    return util.run_command(command=convert, parser=parser, args=args, name='pyomo2nl').retval

def pyomo2nl_main(args=None):
    ans = pyomo2nl(args)
    if ans is None:
        return 0
    return 1

def pyomo2osil(args=None):
    global format
    parser = create_parser('pyomo2osil')
    format = ProblemFormat.osil
    return util.run_command(command=convert, parser=parser, args=args, name='pyomo2osil').retval

def pyomo2osil_main(args=None):
    ans = pyomo2osil(args)
    if ans is None:
        return 0
    return 1




#
# Imports
#
import sys
import os
sys.path.append(".."+os.sep+"../..")
from coopr.pyomo import *

##
## Setting up a Model
##
#
# Create the model
#
model = AbstractModel()
#
# Create sets used to define parameters
#
model.A = Set()
model.B = Set()

##
## Declaring Params
##
#
#
# A simple parameter
#
model.Z = Param()
#
# A single-dimension parameter
#
model.Y = Param(model.A)
#
# Initializing a parameter with two indices
#
model.X = Param(model.A,model.B)
   
##
## Parameter Data
##
#
# A parameter can be constructed with the _initialize_ option, which is a 
# function that accepts the parameter indices and model and returns the value
# of that parameter element:
#
def W_init(model, i, j):
    #
    # Create the value of model.W[i,j]
    #
    return i*j
model.W = Param(model.A, model.B, initialize=W_init)
#
# Note that the parameter model.W is not created when this object is
# constructed.  Instead, W_init() is called during the construction of a
# problem instance.
#
# The _initialize_ option can also be used to specify the values in
# a parameter.  These default values may be overriden by later construction
# steps, or by data in an input file:
#
V_init={}
V_init[1]=1
V_init[2]=2
V_init[3]=9
model.V = Param(model.B, initialize=V_init)
#
# Note that parameter V is initialized with a dictionary, which maps 
# tuples from parameter indices to parameter values.  Simple, unindexed
# parameters can be initialized with a scalar value.
#
model.U = Param(initialize=9.9)
#
# Validation of parameter data is supported in two different ways.  First, 
# the domain of feasible parameter values can be specified with the _within_
# option:
#
model.T = Param(within=model.B)
#
# Note that the default domain for parameters is Reals, the set of floating
# point values.
#
# Validation of parameter data can also be performed with the _validate_ 
# option, which is a function that returns True if a parameter value is valid:
#
def S_validate(model, value):
    return value in model.A
model.S = Param(validate=S_validate)

##
## Default Values
##
#
# Pyomo assumes that parameter values are specified in a sparse manner.  For
# example, the instance Param(model.A,model.B) declares a parameter indexed
# over sets A and B.  However, not all of these values are necessarily
# declared in a model.  The default value for all parameters not declared
# is zero. This default can be overriden with the _default_ option.
#
# The following example illustrates how a parameter can be declared where
# every parameter value is nonzero, but the parameter is stored with a sparse
# representation.
#
R_init={}
R_init[2,1]=1
R_init[2,2]=1
R_init[2,3]=1
model.R = Param(model.A, model.B, default=99.0, initialize=R_init)
#
# Note that the parameter default value can also be specified in an input 
# file.  See data.dat for an example.
#
# Note that the explicit specification of a zero default changes Pyomo
# behavior.  For example, consider:
#
#   model.a = Param(model.A, default=0.0)
#   model.b = Param(model.A)
#
# When model.a[x] is accessed and the index has not been explicitly initialized,
# the value zero is returned.  This is true whether or not the parameter has
# been initialized with data.  Thus, the specification of a default value
# makes the parameter seem to be densely initialized.
#
# However, when model.b[x] is accessed and the
# index has not been initialized, an error occurs (and a Python exception is
# thrown).  Since the user did not explicitly declare a default, Pyomo 
# treats the reference to model.b[x] as an error.
#

##
## Process an input file and confirm that we get appropriate 
## parameter instances.
##
instance = model.create("param.dat")
instance.pprint()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import pyutilib.component.core
from coopr.pyomo.base import IModelTransformation
from coopr.pyomo.base import Var
from coopr.pyomo import BooleanSet, IntegerSet, RangeSet, Reals


class RelaxIntegrality(pyutilib.component.core.SingletonPlugin):
    """
    This plugin relaxes integrality in a Pyomo model.
    """

    pyutilib.component.core.implements(IModelTransformation)

    def __init__(self, **kwds):
        kwds['name'] = "relax_integrality"
        pyutilib.component.core.Plugin.__init__(self, **kwds)

    def apply(self, model, **kwds):
        #
        # Clone the model
        #
        M = model.clone()
        #
        # Iterate over all variables, replacing the domain with a real-valued domain
        # and setting appropriate bounds.
        #
        # Note: this technique is not extensible.  If a user creates a new integer variable
        # class then we'd need to add it here (unless it was a subset of one of the sets we
        # detect here.  Alternatively, we could make the variables responsible for managing
        # the generation of a relaxed variable.  But that would pollute the variable objects
        # with logic about generating a specific relaxation.  That's probably a worse alternative.
        #
        # TODO: rework this so it works with model instances
        #
        comp = M.components[Var]
        for var in comp.values():
            if isinstance(var.domain, BooleanSet):
                var.domain=Reals
                dbnd = ( 0.0, 1.0 )
            elif isinstance(var.domain, IntegerSet):
                dbnd = var.domain.bounds()
                if dbnd is None:
                    bnd = ( None, None )
                var.domain=Reals
            elif isinstance(var.domain, RangeSet):
                dbnd = var.domain.bounds()
                if dbnd is None:
                    bnd = ( None, None )
                var.domain=Reals
            else:
                continue

            bnd = var.bounds
            if bnd is None:
                bnd = ( None, None )

            bnd = ( self._tightenBound(bnd[0], dbnd[0], max),
                    self._tightenBound(bnd[1], dbnd[1], min) )
            if bnd == (None, None):
                var.bounds = None
            else:
                var.bounds = bnd
        return M

    def _tightenBound(self, a, b, comp):
        if a is None:
            return b
        if b is None:
            return a
        return comp(a,b)



#
# Unit Tests for components
#

import unittest
import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
from coopr.pyomo import *
import coopr.pyomo.base.component


class PyomoModel(unittest.TestCase):

    def test_component1(self):
        try:
            Component()
            self.fail("Expected DeveloperError")
        except coopr.pyomo.base.component.DeveloperError:
            pass

if __name__ == "__main__":
    unittest.main()



import pmedian

def pyomo_preprocess(**kwds):
    print "PREPROCESSING",kwds.keys()

def pyomo_create_model(**kwds):
    print "CREATING MODEL",kwds.keys()
    return pmedian.model

def pyomo_print_model(**kwds):
    print "PRINTING MODEL",kwds.keys()

def pyomo_print_instance(**kwds):
    print "PRINTING INSTANCE",kwds.keys()

def pyomo_save_instance(**kwds):
    print "SAVE INSTANCE",kwds.keys()

def pyomo_print_results(**kwds):
    print "PRINTING RESULTS",kwds.keys()

def pyomo_save_results(**kwds):
    print "SAVING RESULTS",kwds.keys()

def pyomo_postprocess(**kwds):
    print "POSTPROCESSING",kwds.keys()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['IPyomoExpression', 'ExpressionFactory', 'ExpressionRegistration',
        'IPyomoSet', 'IModelComponent',
        'ModelComponentFactory',
        'IPyomoPresolver', 'IPyomoPresolveAction',
        'DataManagerFactory',
        'IParamRepresentation',
        'ParamRepresentationFactory',
        'IModelTransformation',
        'apply_transformation',
        'IPyomoScriptPreprocess',
        'IPyomoScriptCreateModel',
        'IPyomoScriptCreateModelData',
        'IPyomoScriptModifyInstance',
        'IPyomoScriptPrintModel',
        'IPyomoScriptPrintInstance',
        'IPyomoScriptSaveInstance',
        'IPyomoScriptPrintResults',
        'IPyomoScriptSaveResults',
        'IPyomoScriptPostprocess'
        ]

from pyutilib.component.core import *


PluginGlobals.push_env('coopr.pyomo.scripting')

class IPyomoScriptPreprocess(Interface):

    def apply(self, **kwds):
        """Apply preprocessing step in the Pyomo script"""

class IPyomoScriptCreateModel(Interface):

    def apply(self, **kwds):
        """Apply model creation step in the Pyomo script"""

class IPyomoScriptModifyInstance(Interface):

    def apply(self, **kwds):
        """Modify and return the model instance"""

class IPyomoScriptCreateModelData(Interface):

    def apply(self, **kwds):
        """Apply model data creation step in the Pyomo script"""

class IPyomoScriptPrintModel(Interface):

    def apply(self, **kwds):
        """Apply model printing step in the Pyomo script"""

class IPyomoScriptPrintInstance(Interface):

    def apply(self, **kwds):
        """Apply instance printing step in the Pyomo script"""

class IPyomoScriptSaveInstance(Interface):

    def apply(self, **kwds):
        """Apply instance saving step in the Pyomo script"""

class IPyomoScriptPrintResults(Interface):

    def apply(self, **kwds):
        """Apply results printing step in the Pyomo script"""

class IPyomoScriptSaveResults(Interface):

    def apply(self, **kwds):
        """Apply results saving step in the Pyomo script"""

class IPyomoScriptPostprocess(Interface):

    def apply(self, **kwds):
        """Apply postprocessing step in the Pyomo script"""

PluginGlobals.pop_env()

class IPyomoPresolver(Interface):

    def get_actions(self):
        """Return a list of presolve actions, in the order in which
        they will be applied."""

    def activate_action(self, action):
        """Activate an action, but leave its default rank"""

    def deactivate_action(self, action):
        """Deactivate an action"""

    def set_actions(self, actions):
        """Set presolve action list"""

    def presolve(self, instance):
        """Apply the presolve actions to this instance, and return the
        revised instance"""


class IPyomoPresolveAction(Interface):

    def presolve(self, instance):
        """Apply the presolve action to this instance, and return the
        revised instance"""

    def rank(self):
        """Return an integer that is used to automatically order presolve actions,
        from low to high rank."""


class IPyomoExpression(Interface):

    def type(self):
        """Return the type of expression"""

    def create(self, args):
        """Create an instance of this expression type"""


class ExpressionRegistration(Plugin):

    implements(IPyomoExpression, service=False)

    def __init__(self, type, cls, swap=False):
        self._type = type
        self._cls = cls
        self._swap = swap

    def type(self):
        return self._type

    def create(self, args):
        if self._swap:
            args = list(args)
            args.reverse()
        return self._cls(args)

def ExpressionFactory(name=None, args=[]):
    ep = ExpressionFactory.ep
    if name is None:
        return map(lambda x:x.name, ep())
    return ep.service(name).create(args)
ExpressionFactory.ep = ExtensionPoint(IPyomoExpression)



class IPyomoSet(Interface):
    pass


class IModelComponent(Interface):
    pass


ModelComponentFactory = CreatePluginFactory(IModelComponent)


class IDataManager(Interface):

    def initialize(self, filename, **kwds):
        """ Prepare to read a data file. """
        pass

    def open(self):
        """ Open the data file. """
        pass

    def close(self):
        """ Close the data file. """
        pass

    def read(self):
        """ Read the data file. """
        pass

    def process(self, model, data, default):
        """ Process the data. """
        pass

    def clear(self):
        """ Reset Plugin. """
        pass

DataManagerFactory = CreatePluginFactory(IDataManager)


class IModelTransformation(Interface):

    def apply(self, model, **kwds):
        """Apply a model transformation and return a new model instance"""

    def __call__(self, model, **kwds):
        """Use this plugin instance as a functor to apply a transformation"""
        return self.apply(model, **kwds)


def apply_transformation(*args, **kwds):
    ep = ExtensionPoint(IModelTransformation)
    if len(args) is 0:
        return map(lambda x:x.name, ep())
    service = ep.service(args[0])
    if len(args) == 1 or service is None:
        return service
    tmp=(args[1],)
    return service.apply(*tmp, **kwds)


class IParamRepresentation(Interface):
    pass

ParamRepresentationFactory = CreatePluginFactory(IParamRepresentation)



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['display']

import logging
import sys

logger = logging.getLogger('coopr.pyomo')


def display(obj, ostream=None):
    """ Display data in a Pyomo object"""
    if ostream is None:
        ostream = sys.stdout
    try:
        obj.display(ostream=ostream)
    except Exception, err:
        raise TypeError, "Error trying to display values for object of type "+str(type(obj))+": "+str(err)


def create_name(name, ndx):
    if ndx is None:
        return name
    if type(ndx) is tuple:
        tmp = str(ndx).replace(', ',',')
        return name+"["+tmp[1:-1]+"]"
    return name+"["+str(ndx)+"]"


def apply_indexed_rule(obj, rule, model, index):
    if index.__class__ is tuple:
        return rule(model, *index)
    return rule(model, index)

def apply_parameterized_indexed_rule(obj, rule, model, param, index):
    if index.__class__ is tuple:
        return rule(model, param, *index)
    return rule(model, param, index)




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

#
# Problem Writer for CPLEX LP Format Files
#

import itertools
import math
import weakref

from coopr.opt import ProblemFormat
from coopr.opt.base import AbstractProblemWriter, SymbolMap
from coopr.pyomo.base import BooleanSet, Constraint, ConstraintList, expr, IntegerSet, Component
from coopr.pyomo.base import active_components_generator, active_subcomponents_generator
from coopr.pyomo.base import Var, value, label_from_name, NumericConstant
from coopr.pyomo.base.sos import SOSConstraint
from coopr.pyomo.base.objective import Objective, minimize, maximize

from pyutilib.component.core import alias
from pyutilib.misc import tostr, PauseGC

class CPXLP_numeric_labeler(object):

    def __init__(self, prefix):

        self.id = 0
        self.prefix = prefix

    def __call__(self, obj):

        self.id += 1
        return self.prefix + str(self.id)

class CPXLP_text_labeler(object):

    def __call__(self, obj):

        try:
            # the most common input object is something that is an element
            # of an indexed component - in this case, we're looking for 
            # the parent component, to try to chase up the block chain.
            block = obj.component()
        except:
            # if we can't find a "component" attribute, then this might be 
            # an indexed or non-indexed component - rather than a part of such
            # a compoent.d in this case, we just assign the "block" (which is 
            # a terrible name in this context) to the current component. 
            # subsequent logic in this function will cause the parent-chasing
            # to occur.
            if isinstance(obj, Component):
                block = obj
            else:
                block = None

        if block is None:
            return label_from_name(str(obj.name))
        block = block._parent
        if block is None:
            return label_from_name(str(obj.name))

        label = str(obj.name)

        block = block()
        while block._parent is not None and block._parent() is not None:
            label = block.name + "." + label
            block = block._parent()
        return label_from_name(label)

class ProblemWriter_cpxlp(AbstractProblemWriter):

    alias('cpxlp')
    alias('lp')

    def __init__(self):

        AbstractProblemWriter.__init__(self, ProblemFormat.cpxlp)

        # the LP writer is responsible for tracking which variables are referenced in constraints,
        # so that one doesn't end up with a zillion "unreferenced variables" warning messages.
        # stored at the object level to avoid additional method arguments. collection of id(_VarData).
        self._referenced_variable_ids = set()

    def __call__(self, model, output_filename, solver_capability, generate_symbolic_labels):

        # when sorting, there are a non-trivial number of temporary objects
        # created. these all yield non-circular references, so disable GC -
        # the overhead is non-trivial, and because references are non-circular,
        # everything will be collected immediately anyway.
        suspend_gc = PauseGC()

        # clear the collection of referenced variables.
        self._referenced_variable_ids.clear()

        if output_filename is None:
            output_filename = model.name + ".lp"

        output_file=open(output_filename, "w")
        symbol_map = self._print_model_LP(model, output_file, solver_capability, generate_symbolic_labels)
        output_file.close()

        return output_filename, symbol_map

    def _get_bound(self, exp):

        if isinstance(exp,expr._IdentityExpression):
            return self._get_bound(exp._args[0])
        elif exp.is_fixed():
            return exp()
        else:
            raise ValueError, "ERROR: non-fixed bound: " + str(exp)
            return None

    def _print_expr_linear(self, x, output_file, object_symbol_dictionary, is_objective):

        """
        Return a expression as a string in LP format.

        Note that this function does not handle any differences in LP format
        interpretation by the solvers (e.g. CPlex vs GLPK).  That decision is
        left up to the caller.

        required arguments:
          x: A Pyomo linear encoding of an expression to write in LP format
        """

        # Currently, it appears that we only need to print the constant
        # offset term for objectives.
        print_offset = is_objective

        constant_term = x[0]
        linear_terms = x[1]

        name_to_coefficient_map = {}

        for coefficient, var_value in linear_terms:

            if var_value.fixed is True:

                constant_term += (coefficient * var_value.value)

            else:

                var_id = id(var_value)
                if var_id not in self._referenced_variable_ids:
                    self._referenced_variable_ids.add(var_id)

                name = object_symbol_dictionary[id(var_value)]

                # due to potential disabling of expression simplification,
                # variables might appear more than once - condense coefficients.
                name_to_coefficient_map[name] = coefficient + name_to_coefficient_map.get(name,0.0)

        sorted_names = sorted(name_to_coefficient_map.keys())

        for name in sorted_names:

            coefficient = name_to_coefficient_map[name]

            sign = '+'
            if coefficient < 0: sign = '-'
            print >>output_file, '%s%f %s' % (sign, math.fabs(coefficient), name)

        if print_offset and (constant_term != 0.0):
            sign = '+'
            if constant_term < 0: sign = '-'
            print >>output_file, '%s%f %s' % (sign, math.fabs(constant_term), 'ONE_VAR_CONSTANT')

        return constant_term


    def _print_expr_canonical(self, x, output_file, object_symbol_dictionary, is_objective):

        """
        Return a expression as a string in LP format.

        Note that this function does not handle any differences in LP format
        interpretation by the solvers (e.g. CPlex vs GLPK).  That decision is
        left up to the caller.

        required arguments:
          x: A Pyomo canonical expression to write in LP format
        """

        # cache - this is referenced numerous times.
        var_hashes = x[-1]

        #
        # Linear
        #
        if 1 in x:

            for var_hash in x[1]:
                var_id = id(var_hashes[var_hash])
                if var_id not in self._referenced_variable_ids:
                    self._referenced_variable_ids.add(var_id)            

            sorted_names = [(object_symbol_dictionary[id(var_hashes[var_hash])], var_coefficient) for var_hash, var_coefficient in x[1].iteritems()]
            sorted_names.sort()

            for name, coef in sorted_names:
                # there are various situations in which the coefficients may be equal to 0.0,
                # in which case there is no real reason to output the corresponding terms.
                if coef: # != 0.0:
                    print >>output_file, '%+f' % coef, name

        #
        # Quadratic
        #
        if 2 in x:

            # first, make sure there is something to output - it is possible for all
            # terms to have coefficients equal to 0.0, in which case you don't want
            # to get into the bracket notation at all.
            # NOTE: if the coefficient is really 0.0, it should be preprocessed out by
            #       the canonial expression generator!
            found_nonzero_term = False # until proven otherwise
            for var_hash, var_coefficient in x[2].iteritems():
                for var in var_hash:
                    var_value = var_hashes[var]
                    var_id = id(var_value)
                    if var_id not in self._referenced_variable_ids:
                        self._referenced_variable_ids.add(var_id)
                    
                if math.fabs(var_coefficient) != 0.0:
                    found_nonzero_term = True
                    break

            if found_nonzero_term:

                print >>output_file, "+ ["

                num_output = 0

                for var_hash in sorted(x[2].keys()):

                    coefficient = x[2][var_hash]
                    sign = '+'
                    if coefficient < 0:
                        sign = '-'
                        coefficient = math.fabs(coefficient)

                    if is_objective:
                        coefficient *= 2
                    # times 2 because LP format requires /2 for all the quadratic
                    # terms /of the objective only/.  Discovered the last bit thru
                    # trial and error.  Obnoxious.
                    # Ref: ILog CPlex 8.0 User's Manual, p197.

                    print >>output_file, sign, coefficient,

                    term_variables = []

                    for var in var_hash:
                        var_value = var_hashes[var]
                        name = object_symbol_dictionary[id(var_value)]
                        term_variables.append(name)

                    if len(term_variables) == 2:
                        print >>output_file, term_variables[0],"*",term_variables[1],
                    else:
                        print >>output_file, term_variables[0],"^ 2",
                    print >>output_file, ""

                print >>output_file, ""

                print >>output_file, "]",

                if is_objective:
                    print >>output_file, ' / 2'
                    # divide by 2 because LP format requires /2 for all the quadratic
                    # terms.  Weird.  Ref: ILog CPlex 8.0 User's Manual, p197
                else:
                    print >>output_file, ""


        #
        # Constant offset
        #
        if 0 in x:
            offset = x[0][None]
        else:
            offset=0.0

        # Currently, it appears that we only need to print the constant
        # offset term for objectives.
        if is_objective and offset != 0.0:
            print >>output_file, '%s%f %s' % (offset < 0 and '-' or '+', math.fabs(offset), 'ONE_VAR_CONSTANT')

        #
        # Return constant offset
        #
        return offset

    @staticmethod
    def printSOS(symbol_map,labeler, con, name, output_file, index=None):

        """
        Returns the SOS constraint (as a string) associated with con.
        If specified, index is passed to con.sos_set().

        Arguments:
        con    The SOS constraint object
        name   The name of the variable
        output_file The output stream
        index  [Optional] the index to pass to the sets indexing the variables.
        """

        # The name of the variable being indexed
        var = con.sos_vars()

        # The list of variable names to be printed, including indices
        varNames = []

        # Get all the variables
        if index is None:
            tmpSet = con.sos_set()
        else:
            tmpSet = con.sos_set()[index]
        for idx in tmpSet:
            varNames.append(symbol_map.getSymbol(var[idx],labeler))

        
        conNameIndex = ""
        if index is not None:
            if type(index) is tuple:
                conNameIndex += label_from_name(tostr(index))
            else:
                conNameIndex += label_from_name(str(index))

        print >>output_file, '%s_%s: S%s::' % (symbol_map.getSymbol(con,labeler), conNameIndex, con.sos_level())

        # We need to 'weight' each variable
        # For now we just increment a counter
        for i in range(0, len(varNames)):
            print >>output_file, '%s:%f' % (varNames[i], i+1)

    #
    # a simple utility to pass through each variable and constraint
    # in the input model and populate the symbol map accordingly. once
    # we know all of the objects in the model, we can then "freeze"
    # the contents and use more efficient query mechanisms on simple
    # dictionaries - and avoid checking and function call overhead.
    #
    def _populate_symbol_map(self, model, symbol_map, labeler):

        # NOTE: we use createSymbol instead of getSymbol because we know
        # whether or not the symbol exists, and don't want to the overhead
        # of error/duplicate checking.

        # cache frequently called functions
        create_symbol_func = SymbolMap.createSymbol
        create_symbols_func = SymbolMap.createSymbols
        alias_symbol_func = SymbolMap.alias

        for block in model.all_blocks():

            active_objectives = model.active_components(Objective)
            # all blocks don't necessarily have an objective.
            if len(active_objectives) > 0:
               objective = active_objectives[active_objectives.keys()[0]]
               objective_data = objective[objective.keys()[0]]
               create_symbol_func(symbol_map, objective_data, labeler)
               alias_symbol_func(symbol_map, objective_data, '__default_objective__')

            for constraint_data in itertools.chain(active_subcomponents_generator(block, Constraint), 
                                                   active_subcomponents_generator(block, ConstraintList)):
                constraint_data_symbol = create_symbol_func(symbol_map, constraint_data, labeler)
                if constraint_data._equality:
                    label = 'c_e_' + constraint_data_symbol + '_'
                    alias_symbol_func(symbol_map, constraint_data, label)
                else:
                    if constraint_data.lower is not None:
                        label = 'c_l_' + constraint_data_symbol + '_'
                        alias_symbol_func(symbol_map, constraint_data, label)
                    if constraint_data.upper is not None:
                        label = 'c_u_' + constraint_data_symbol + '_'
                        alias_symbol_func(symbol_map, constraint_data, label)

            create_symbols_func(symbol_map, active_subcomponents_generator(block, Var), labeler)

            # TBD - MOVE TO ABOVE, WITH OTHER CONSTRAINTS - ASSUMING THEY HAVE SUB_COMPONENTS?
            sos_con_list = block.active_components(SOSConstraint)
            for con in sos_con_list.itervalues():
                create_symbol_func(symbol_map, con, labeler)

    def _print_model_LP(self, model, output_file, solver_capability, generate_symbolic_labels):

        symbol_map = SymbolMap(model)

        if generate_symbolic_labels is True:
            labeler = CPXLP_text_labeler()
        else:
            labeler = CPXLP_numeric_labeler('x')

        # populate the symbol map in a single pass.
        self._populate_symbol_map(model, symbol_map, labeler)

        # and extract the information we'll need for rapid labeling.
        object_symbol_dictionary = symbol_map.getByObjectDictionary()

        # cache - these are called all the time.
        print_expr_linear = self._print_expr_linear
        print_expr_canonical = self._print_expr_canonical

        #
        # Objective
        #
        _obj = model.active_components(Objective)

        supports_quadratic = solver_capability('quadratic')

        if len(_obj) == 0:
            msg = "ERROR: No objectives defined for input model '%s'; "    \
                  ' cannot write legal LP file'
            raise ValueError, msg % str( model.name )
        if len(_obj) > 1:
            msg = "More than one objective defined for input model '%s'; " \
                  'Cannot write legal LP file\n'                           \
                  'Objectives: %s'
            raise ValueError(
                msg % ( model.name,', '.join("'%s'" % x for x in _obj) ))

        obj = _obj[ _obj.keys()[0] ]
        if obj.sense == maximize:
            print >>output_file, "max "
        else:
            print >>output_file, "min "

        obj_keys = obj.keys()
        if len(obj_keys) > 1:
            keys = obj.name + ' (indexed by: %s)' % ', '.join(map(str, obj_keys))

            msg = "More than one objective defined for input model '%s'; " \
                  'Cannot write legal LP file\n'                           \
                  'Objectives: %s'
            raise ValueError, msg % ( str(model.name), keys )

        obj_data = obj[obj_keys[0]]

        if obj_data.repn is None:
            raise RuntimeError, "No canonical representation identified for objective="+obj_data.name
        if None in obj_data.repn:
            degree = None
        else:
            degree = max(obj_data.repn.iterkeys())

        if degree == 0:
            print ("Warning: Constant objective detected, replacing " +
                   "with a placeholder to prevent solver failure.")

            print >>output_file, object_symbol_dictionary[id(obj_data)] \
                 + ": +0.0 ONE_VAR_CONSTANT"

            # Skip the remaining logic of the section
        else:
            if degree == 2:

                if not supports_quadratic:
                    raise RuntimeError(
                        'Selected solver is unable to handle objective functions with quadratic terms. ' \
                        'Objective at issue: %s.' % obj.name)

            elif degree != 1:
                msg  = "Cannot write legal LP file.  Objective '%s%s' "  \
                       'has nonlinear terms that are not quadratic.'
                if obj_keys[0] is None:
                    msg %= (obj.name, '')
                else:
                    msg %= (obj.name, '[%s]' % obj_keys[0] )
                raise RuntimeError, msg

            print >>output_file, object_symbol_dictionary[id(obj_data)]+':'

            offset = print_expr_canonical( obj_data.repn,
                                                 output_file,
                                                 object_symbol_dictionary,
                                                 True )

            print >>output_file, ""

        # Constraints
        #
        # If there are no non-trivial constraints, you'll end up with an empty
        # constraint block. CPLEX is OK with this, but GLPK isn't. And
        # eliminating the constraint block (i.e., the "s.t." line) causes GLPK
        # to whine elsewhere. output a warning if the constraint block is empty,
        # so users can quickly determine the cause of the solve failure.

        print >>output_file, "s.t."
        print >>output_file, ""

        have_nontrivial = False

        # FIXME: This is a hack to get nested blocks working...
        for block in model.all_blocks():
            active_constraints = block.active_components(Constraint)
            active_constraintlists = block.active_components(ConstraintList)
            for constraint in itertools.chain(active_constraints.itervalues(), active_constraintlists.itervalues()):
                if constraint.trivial:
                    continue

                have_nontrivial=True

                for index in sorted(constraint.keys()):

                    constraint_data = constraint[index]
                    if not constraint_data.active:
                        continue

                    # if expression trees have been linearized, then the canonical
                    # representation attribute on the constraint data object will
                    # be equal to None.
                    if (constraint_data.repn is not None):

                        if None in constraint_data.repn:
                            degree = None
                        else:
                            degree = max(constraint_data.repn.iterkeys())

                        # There are conditions, e.g., when fixing variables, under which
                        # a constraint block might be empty.  Ignore these, for both
                        # practical reasons and the fact that the CPLEX LP format
                        # requires a variable in the constraint body.  It is also
                        # possible that the body of the constraint consists of only a
                        # constant, in which case the "variable" of
                        if degree == 0:
                            # this happens *all* the time in many applications,
                            # including PH - so suppress the warning.
                            #
                            #msg = 'WARNING: ignoring constraint %s[%s] which is ' \
                            #      'constant'
                            #print msg % (str(C),str(index))
                            continue

                        if degree == 2:
                            if not supports_quadratic:
                                msg  = 'Solver unable to handle quadratic expressions.'\
                                       "  Constraint at issue: '%s%%s'"
                                msg %= constraint.name
                                if index is None: msg %= ''
                                else: msg %= '[%s]' % index

                                raise ValueError, msg

                        elif degree != 1:
                            msg = "Cannot write legal LP file.  Constraint '%s%s' "   \
                                  'has a body with nonlinear terms.'
                            if index is None:
                                msg %= ( constraint.name, '')
                            else:
                                msg %= ( constraint.name, '[%s]' % index )
                            raise ValueError, msg

                    con_symbol = object_symbol_dictionary[id(constraint_data)]
                    if constraint_data._equality:
                        label = 'c_e_' + con_symbol + '_'
                        print >>output_file, label+':'
                        if constraint_data.lin_body is not None:
                            offset = print_expr_linear(constraint_data.lin_body, output_file, object_symbol_dictionary, False)
                        else:
                            offset = print_expr_canonical(constraint_data.repn, output_file, object_symbol_dictionary, False)
                        bound = constraint_data.lower
                        bound = self._get_bound(bound) - offset
                        print >>output_file, "=", bound
                        print >>output_file, ""
                    else:
                        if constraint_data.lower is not None:
                            label = 'c_l_' + con_symbol + '_'
                            print >>output_file, label+':'
                            if constraint_data.lin_body is not None:
                                offset = print_expr_linear(constraint_data.lin_body, output_file, object_symbol_dictionary, False)
                            else:
                                offset = print_expr_canonical(constraint_data.repn, output_file, object_symbol_dictionary, False)
                            bound = constraint_data.lower
                            bound = self._get_bound(bound) - offset
                            print >>output_file, ">=", bound
                            print >>output_file, ""
                        if constraint_data.upper is not None:
                            label = 'c_u_' + con_symbol + '_'
                            print >>output_file, label+':'
                            if constraint_data.lin_body is not None:
                                offset = print_expr_linear(constraint_data.lin_body, output_file, object_symbol_dictionary, False)
                            else:
                                offset = print_expr_canonical(constraint_data.repn, output_file, object_symbol_dictionary, False)
                            bound = constraint_data.upper
                            bound = self._get_bound(bound) - offset
                            print >>output_file, "<=", bound
                            print >>output_file, ""

        if not have_nontrivial:
            print 'WARNING: Empty constraint block written in LP format '  \
                  '- solver may error'

        # the CPLEX LP format doesn't allow constants in the objective (or
        # constraint body), which is a bit silly.  To avoid painful
        # book-keeping, we introduce the following "variable", constrained
        # to the value 1.  This is used when quadratic terms are present.
        # worst-case, if not used, is that CPLEX easily pre-processes it out.
        prefix = ""
        print >>output_file, '%sc_e_ONE_VAR_CONSTANT: ' % prefix
        print >>output_file, '%sONE_VAR_CONSTANT = 1.0' % prefix
        print >>output_file, ""

        #
        # Bounds
        #

        print >>output_file, "bounds "

        # Scan all variables even if we're only writing a subset of them.
        # required because we don't store maps by variable type currently.

        # Track the number of integer and binary variables, so you can
        # output their status later.
        niv = nbv = 0

        # FIXME: This is a hack to get nested blocks working...
        for block in model.all_blocks():
            for var_value in active_subcomponents_generator(block, Var, sort_by_keys=True):

                if (not var_value.active) or (var_value.fixed is True) or (id(var_value) not in self._referenced_variable_ids):
                    continue

                # track the number of integer and binary variables, so we know whether
                # to output the general / binary sections below.
                if isinstance(var_value.domain, IntegerSet):   niv += 1
                elif isinstance(var_value.domain, BooleanSet): nbv += 1

                # in the CPLEX LP file format, the default variable
                # bounds are 0 and +inf.  These bounds are in
                # conflict with Pyomo, which assumes -inf and +inf
                # (which we would argue is more rational).
                print >>output_file,"   ",
                if var_value.lb is not None:
                    print >>output_file, value(var_value.lb), "<= ",
                else:
                    print >>output_file, " -inf <= ",
                name_to_output = object_symbol_dictionary[id(var_value)]
                if name_to_output == "e":
                    msg = 'Attempting to write variable with name' \
                        "'e' in a CPLEX LP formatted file - "    \
                        'will cause a parse failure due to '     \
                        'confusion with numeric values '         \
                        'expressed in scientific notation'
                    raise ValueError, msg
                print >>output_file, name_to_output,
                if var_value.ub is not None:
                    print >>output_file, " <=", value(var_value.ub)
                else:
                    print >>output_file, " <= +inf"

        if niv > 0:

            print >>output_file, "general"

            for block in model.all_blocks():
                for variable in active_components_generator(block, Var, sort_by_keys=True):
                    for index in sorted(variable.integer_keys()):
                        var_value = variable[index]
                        if (not var_value.active) or (var_value.fixed is True) or (id(var_value) not in self._referenced_variable_ids):
                            continue
                        var_name = object_symbol_dictionary[id(var_value)]
                        print >>output_file, ' ', var_name

        if nbv > 0:

            print >>output_file, "binary"

            for block in model.all_blocks():
                for variable in active_components_generator(block, Var, sort_by_keys=True):
                    for index in sorted(variable.binary_keys()):
                        var_value = variable[index]
                        if (not var_value.active) or (var_value.fixed is True) or (id(var_value) not in self._referenced_variable_ids):
                            continue
                        var_name = object_symbol_dictionary[id(var_value)]
                        print >>output_file, ' ', var_name


        # SOS constraints
        #
        # For now, we write out SOS1 and SOS2 constraints in the cplex format
        #
        # All Component objects are stored in model._component, which is a
        # dictionary of {class: {objName: object}}.
        #
        # Consider the variable X,
        #
        #   model.X = Var(...)
        #
        # We print X to CPLEX format as X(i,j,k,...) where i, j, k, ... are the
        # indices of X.
        #
        # TODO: Allow users to specify the variables coefficients for custom
        # branching/set orders - also update ampl.py, CPLEXDirect.py, gurobi_direct.py
        sosn = solver_capability("sosn")
        sos1 = solver_capability("sos1")
        sos2 = solver_capability("sos2")
        writtenSOS = False
        for block in model.all_blocks():
            sos_con_list = block.active_components(SOSConstraint)

            if len(sos_con_list) > 0:
                if not(sos1 or sos2 or sosn):
                    raise Exception(
                        "Solver does not support SOSConstraint declarations")

            for con in sos_con_list.itervalues():
                level = con.sos_level()
                if (level == 1 and not sos1) or (level == 2 and not sos2) or \
                        (level > 2 and not sosn):
                    raise Exception(
                        "Solver does not support SOS level %s constraints"
                        % (level,) )
                if writtenSOS == False:
                    print >>output_file, "SOS"
                    writtenSOS = True
                name = symbol_map.getSymbol( con, labeler )
                masterIndex = con.sos_set_set()
                if None in masterIndex:
                    # A single constraint
                    self.printSOS(symbol_map, labeler, con, name, output_file)
                else:
                    # A series of indexed constraints
                    for index in masterIndex:
                        self.printSOS(symbol_map, labeler, con, name, output_file, index)


        #
        # wrap-up
        #
        print >>output_file, "end "

        return symbol_map



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Param']

import sys
import types
import logging
import weakref

from pyutilib.component.core import alias
import pyutilib.math

from component import Component
from sparse_indexed_component import SparseIndexedComponent
from indexed_component import IndexedComponent
from misc import apply_indexed_rule, apply_parameterized_indexed_rule, \
     create_name
from numvalue import NumericConstant, NumericValue, native_types
from set_types import Any, Reals

from numvalue import value

logger = logging.getLogger('coopr.pyomo')

class _ParamData(NumericConstant):
    """Holds the numeric value of a mutable parameter"""

    __slots__ = []

    def __init__(self, name, domain, value):
        """Constructor"""
        #
        # NumericConstant.__init__(self, name, domain, value)
        #
        if domain is None:
            self.domain = Reals
        else:
            self.domain = domain
        self.name = name
        if pyutilib.math.is_nan(value):
            self.value = pyutilib.math.nan
        else:
            self.value = value


    def __str__(self):
        return str(self.name)

    def is_constant(self):
        return False

    def clear(self):
        self.value = None


class Param(SparseIndexedComponent):
    """A parameter value, which may be defined over a index"""

    """ Constructor
        Arguments:
           name       The name of this parameter
           index      The index set that defines the distinct parameters.
                         By default, this is None, indicating that there
                         is a single parameter.
           within     A set that defines the type of values that
                         each parameter must be.
           validate   A rule for validating this parameter w.r.t. data
                         that exists in the model
           default    A set that defines default values for this parameter
           rule       A rule for setting up this parameter with existing model
                         data
           nochecking If true, various checks regarding valid domain and index
                      values are skipped. Only intended for use by algorithm
                      developers - not modelers!
    """

    alias( "Param",
           doc="Parameter data that is used to define a model instance.",
           subclass=True )

    def __new__(cls, *args, **kwds):
        if cls != Param:
            return super(Param, cls).__new__(cls)
        if args == ():
            return _ParamElement.__new__(_ParamElement)
        else:
            return _ParamArray.__new__(_ParamArray)

    def __init__(self, *args, **kwd):
        kwd.pop('repn',None)
        
        self._initialize = kwd.pop('initialize', None )
        self._initialize = kwd.pop('rule', self._initialize )
        self._validate   = kwd.pop('validate', None )
        self.domain      = kwd.pop('within', Any )
        self.nochecking  = kwd.pop('nochecking',False)
        self._mutable    = kwd.pop('mutable', False )

        self._default_val = None
        # Use set_default() so that we vaidate the value against the domain
        default = kwd.pop('default', None )
        
        #
        kwd.setdefault('ctype', Param)
        SparseIndexedComponent.__init__(self, *args, **kwd)
        self.set_default(default)
        
        # Because we want to defer the check for defined values until
        # runtime, we will undo the weakref to ourselves
        if not self.is_indexed():
            del self._data[None]


    def pprint(self, ostream=None, verbose=False):
        if ostream is None:
            ostream = sys.stdout
        ostream.write( "  %s : " % (self.name,))
        if self.doc is not None:
            ostream.write("\t%s\n" % (self.doc,))
        ostream.write( "\tSize=%s \tDomain=%s\n"
                       % (len(self), self.domain.name) )
        if not self._constructed:
            ostream.write("\tNot constructed\n")
        elif None in self.keys():
            if None in self._data:
                ostream.write("\t%s\n" % ( value(self._data[None]()), ))
            else:
                ostream.write("\tUndefined\n")
        else:
            for key, val in sorted(self.sparse_iteritems()):
                ostream.write("\t%s : %s\n" % (key, value(val)))
        if self._default_val is not None:
            ostream.write("\tdefault: %s\n" % (value(self._default_val)))

    # TODO: Not sure what "reset" really means in this context...
    def reset(self):
        pass

    #
    # A utility to extract all index-value pairs defining this
    # parameter, returned as a dictionary. useful in many
    # contexts, in which key iteration and repeated __getitem__
    # calls are too expensive to extract the contents of a parameter.
    # NOTE: We are presently not careful if an index has not been
    #       explicitly defined - issues around defaults are bound
    #       to crop up.
    # NOTE: Using this method will cause sparse mutable Params to become
    #       dense!
    #
    def extract_values(self):
        return dict((key,value(val)) for key,val in self.iteritems())

    def sparse_extract_values(self):
        return dict((key,value(val)) for key,val in self.sparse_iteritems())


    def _default(self, idx):
        # FIXME: Ideally, we should test for using an unconstructed
        # Param; however, actually enforcing this breaks lots & lots of
        # tests...
        #        
        #if not self._constructed:
        #    if idx is None:
        #        idx_str = '%s' % (self.name,)
        #    else:
        #        idx_str = '%s[%s]' % (self.name, idx,)
        #    raise ValueError(
        #        "Error retrieving Param value (%s): The Param value has "
        #        "not been constructed" % ( idx_str,) )                
        if self._default_val is None:
            if idx is None:
                idx_str = '%s' % (self.name,)
            else:
                idx_str = '%s[%s]' % (self.name, idx,)
            raise ValueError(
                "Error retrieving Param value (%s): The Param value is "
                "undefined and no default value is specified"
                % ( idx_str,) )
        if self._mutable:
            if self.is_indexed():
                return self._data.setdefault(
                    idx, _ParamData( create_name(self.name,idx), 
                                     self.domain, self._default_val ) )
            else:
                self[None] = self._default_val
                return self
        else:
            return self._default_val

    def set_default(self, val):
        if self._constructed and val is not None and val not in self.domain:
            raise ValueError(
                "Default value (%s) is not valid for Param domain %s" %
                ( str(val), self.domain.name ) )
        self._default_val = val

    def __setitem__(self, ndx, val):
        if self._constructed and not self._mutable:
            raise TypeError(
"""Attempting to set the value of the immutable parameter %s after the
parameter has been constructed.  If you intend to change the value of
this parameter dynamically, please declare the parameter as mutable
[i.e., Param(mutable=True)]""" % (self.name,))

        # Params should contain *values*.  Normally, I would just call
        # value(), but that forces the value to be a /numeric value/,
        # which for historical reasons, we have not forced.  Notably, we
        # have allowed Params with domain==Any to hold strings, tuples,
        # etc.  The following lets us use NumericValues to initialize
        # Params, but is optimized to check for "known" native types to
        # bypass a potentially expensive isinstance()==False call. 
        #
        if val.__class__ not in native_types:
            if isinstance(val, NumericValue):
                val = val()

        #
        # Validate the index
        #

        if self.nochecking:
            # Ironically, if we are *not* checking index values, then we
            # *must* flatten the incoming index tuple
            ndx = self.normalize_index(ndx)

        #
        # TBD: Potential optimization: if we find that updating a Param is
        # more common than setting it in the first place, then first
        # checking the _data and then falling back on the _index *might*
        # be more efficient.
        #
        elif ndx not in self._index:

            # We rely (for performance purposes) on "most" people doing things 
            # correctly; that is, they send us either a scalar or a valid tuple.  
            # So, for efficiency, we will check the index *first*, and only go
            # through the hassle of flattening things if the ndx is not found.
            ndx = self.normalize_index(ndx)
            if ndx not in self._index:
                if not self.is_indexed():
                    msg = "Error setting parameter value: " \
                          "Cannot treat the scalar Param '%s' as an array" \
                          % ( self.name, )
                else:
                    msg = "Error setting parameter value: " \
                          "Index '%s' is not valid for array Param '%s'" \
                          % ( ndx, self.name, )
                raise KeyError(msg)

        if ndx is None:
            self.value = val
            self._data[ndx] = weakref.ref(self)
        elif self._mutable:
            if ndx in self._data:
                self._data[ndx].value = val
            else:
                self._data[ndx] = _ParamData(create_name(self.name,ndx), self.domain, val)
        else:
            self._data[ndx] = val
        #
        # This is at the end, so the user can write their validation
        # tests as if the data was already added.
        #
        if self.nochecking:
            return
        if val not in self.domain:
            raise ValueError(
                "Invalid parameter value: %s[%s] = '%s', value type=%s.\n"
                "\tValue not in parameter domain %s" %
                ( self.name, ndx, val, type(val), self.domain.name ) )
        if self._validate is not None:
            if self.is_indexed():
                if type(ndx) is tuple:
                    tmp = ndx
                else:
                    tmp = (ndx,)
            else:
                tmp = ()
            if not apply_parameterized_indexed_rule(
                self, self._validate, self._parent(), val, tmp ):
                raise ValueError(
                    "Invalid parameter value: %s[%s] = '%s', value type=%s.\n"
                    "\tValue failed parameter validation rule" %
                    ( self.name, ndx, val, type(val) ) )

    def construct(self, data=None):
        """ Apply the rule to construct values in this set """

        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Param, name=%s, from data=%s"
                             % ( self.name, `data` ))

        if self._constructed:
            raise IOError(
                "Cannot reconstruct parameter '%s' (already constructed)"
                % self.name )
            return

        self.clear()

        rule = getattr(self,'rule',None)
        if rule is not None:
            self._initialize=rule

        #
        # Construct using the initial data or the data loaded from an
        # external source. Data values will be queried in to following
        # order:
        #   1) the 'data' dictionary
        #   2) the self._initialize dictionary or rule
        #   3) [implicit: fall back on the default value]
        #
        # To accomplish this, we will first set all the values based on
        # self._initialize, and then allow the data to overwrite anything.
        #
        # NB: Singleton Params can always be treated as "normal" indexed
        # params, indexed by a set {None}.
        #

        #
        # NB: Previously, we would raise an exception for constructing
        # scalar parameters with no defined data.  As that was a special
        # case (i.e. didn't apply to arrays) and was frustrating for
        # Concrete folks who were going to initialize the value later,
        # we will allow an undefined Param to be constructed and will
        # instead throw an exception later i the user tries to *use* the
        # Param before it is initialized.
        #

        if self._initialize is not None:
            _init = self._initialize
            _init_type = type(_init)
            if _init_type is dict:
                for key, val in _init.iteritems():
                    self[key] = val
            elif _init_type is types.FunctionType:
                if self.is_indexed():
                    for idx in self:
                        # IMPORTANT: Do *not* call self.__setitem__ here
                        #    - we already know the index is valid (and
                        #    flattened), and validating indices is far too
                        #    expensive to waste time doing it
                        #    again. however, we still need to validate the
                        #    value generated by the rule, relative to the
                        #    parameter domain.
                        # REVISION: given the rewrite of __setitem__,
                        #    the index is not validated unless it is not
                        #    found in the corresponding data / index.  We
                        #    *could* reimplement the optimization metioned
                        #    above if it proves necessary, but until then,
                        #    the DRY principle dictates that we use
                        #    __setitem__.
                        self[idx] = apply_indexed_rule(
                            self, _init, self._parent(), idx )
                else:
                    self[None] = _init(self._parent())
            else:
                if isinstance(_init, NumericValue):
                    # Reduce NumericValues to scalars
                    _init = _init()
                elif isinstance(_init, SparseIndexedComponent):
                    # Ideally, we want to reduce SparseIndexedComponents
                    # to a dict, but without "densifying" it.  However,
                    # since there is no way to (easily) get the default
                    # value, we will take the "less surprising" route of
                    # letting the source become dense, so that we get
                    # the expected copy.
                    sparse_src = len(_init) != len(_init.sparse_keys())
                    tmp = {}
                    for key in _init.keys():
                        try:
                            val = _init[key]
                        except ValueError:
                            continue
                        tmp[key] = value(val)
                    tmp = dict(_init.iteritems())
                    if sparse_src and len(_init) == len(_init.sparse_keys()):
                        sys.stderr.write("""
WARNING: Initializing Param %s using a sparse mutable indexed component (%s).
    This has resulted in the conversion of the source to dense form.
""" % ( self.name, _init.name ) )
                    _init = tmp
                elif isinstance(_init, IndexedComponent):
                    # FIXME: is there a general form for
                    # IndexedComponent, or should we just wait until
                    # everything moves over to SparseIndexedComponents?
                    pass
                # if things looks like a dictionary, then we will treat
                # it as such
                _isDict = '__getitem__' in dir(_init)
                if _isDict:
                    try:
                        for x in _init:
                            _init.__getitem__(x)
                    except:
                        _isDict = False
                if _isDict:
                    for key in _init:
                        self[key] = _init[key]
                else:
                    for key in self._index:
                        self[key] = _init
                
        if data is not None:
            try:
                for key, val in data.iteritems():
                    self[key] = val
            except:
                if type(data) not in [dict]:
                   raise ValueError("Attempting to initialize parameter="+self.name+" with data="+str(data)+". Data type is not a dictionary, and a dictionary is expected. Did you create a dictionary with a \"None\" index?")

        self._constructed = True
        self.set_default(self._default_val)


class _ParamElement(_ParamData, Param):

    def __init__(self, *args, **kwds):
        Param.__init__(self, *args, **kwds)
        _ParamData.__init__(self, self.name, self.domain, kwds.get('default',None))

    def __getstate__(self):
        ans = _ParamData.__getstate__(self)
        ans.update(Param.__getstate__(self))
        return ans

    def __setstate__(self, state):
        for (slot_name, value) in state.iteritems():
            setattr(self, slot_name, value)

    def pprint(self, ostream=None, verbose=False):
        # Needed so that users find Param.pprint and not _ParamData.pprint
        Param.pprint(self, ostream=ostream, verbose=verbose)

    def __call__(self, *args, **kwds):
        # Needed because we rely on self[None] to know if this parameter
        # is valid.  In particular, if we are getting data from the
        # default value, calling self[None] will actually inject the
        # [None] entry into _data.
        ans = self[None]
        if None in self._data:
            return _ParamData.__call__(ans, *args, **kwds)
        else:
            return ans

    def set_value(self, value):
        if self._constructed and not self._mutable:
            raise TypeError(
"""Attempting to set the value of the immutable parameter %s after the
parameter has been constructed.  If you intend to change the value of
this parameter dynamically, please declare the parameter as mutable
[i.e., Param(mutable=True)]""" % (self.name,))
        self[None] = value
        

    def is_constant(self):
        return self._constructed and not self._mutable

class _ParamArray(Param):

    def __init__(self, *args, **kwds):
        Param.__init__(self, *args, **kwds)

    def display(self, ostream=None):
        self.pprint(ostream=ostream)

    def __str__(self):
        return str(self.name)




from coopr.pyomo.check.checker import IterativeDataChecker

import re

class PrintParens(IterativeDataChecker):
    def __init__(self):
        self.current_lineno = 0

    def check(self, runner, script, info):
        self.current_lineno = info[0]
        line = info[1]
        if re.search("print[^\(]", line) is not None:
            self.problem("Print statements in Python 3.x require parentheses", lineno = info[0])

    def checkerDoc(self):
        return """\
        In Python 3, 'print' changed from a language keyword to a function.
        As such, developers need to surround the arguments to 'print' with
        parentheses, e.g.:
            print "Hello"       =>       print("Hello")
        """



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['BuildAction']

import logging
import types

from pyutilib.component.core import alias
from coopr.pyomo.base.indexed_component import IndexedComponent
from coopr.pyomo.base.misc import apply_indexed_rule

logger = logging.getLogger('coopr.pyomo')


class BuildAction(IndexedComponent):
    """A build action, which executes a rule for all valid indices.

    Constructor arguments:
        rule        The rule that is executed for every indice.

    Private class attributes:
        _rule       The rule that is executed for every indice.
    """

    alias("BuildAction", "This component is used to inject arbitrary actions into the model construction process.  The action rule is applied to every index value.")

    def __init__(self, *args, **kwd):
        self._rule = kwd.pop('rule', None)
        kwd['ctype'] = BuildAction
        IndexedComponent.__init__(self, *args, **kwd)
        #
        if not type(self._rule) is types.FunctionType:
            raise ValueError, "BuildAction must have an 'rule' option specified whose value is a function"

    def construct(self, data=None):
        """ Apply the rule to construct values in this set """
        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Action, name="+self.name)
        #
        if self._constructed:
            return
        self._constructed=True
        #
        if None in self._index:
            # Singleton component
            self._rule(self._parent())
        else:
            # Indexed component
            for index in self._index:
                apply_indexed_rule(self, self._rule, self._parent(), index)




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['BuildCheck']

import logging
import types

from pyutilib.component.core import alias
from coopr.pyomo.base.indexed_component import IndexedComponent
from coopr.pyomo.base.misc import apply_indexed_rule

logger = logging.getLogger('coopr.pyomo')


class BuildCheck(IndexedComponent):
    """A build check, which executes a rule for all valid indices.

    Constructor arguments:
        rule         The rule that is executed for every indice.

    Private class attributes:
        _rule       The rule that is executed for every indice.
    """

    alias("BuildCheck", "This component is used to perform a test during the model construction process.  The action rule is applied to every index value, and if the function returns False an exception is raised.")

    def __init__(self, *args, **kwd):
        self._rule = kwd.pop('rule', None)
        kwd['ctype'] = BuildCheck
        IndexedComponent.__init__(self, *args, **kwd)
        #
        if not type(self._rule) is types.FunctionType:
            raise ValueError, "BuildAction must have an 'rule' option specified whose value is a function"

    def construct(self, data=None):
        """ Apply the rule to construct values in this set """
        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Check, name="+self.name)
        #
        if self._constructed:
            return
        self._constructed=True
        #
        if None in self._index:
            # Singleton component
            res = self._rule(self._parent())
            if not res:
                raise ValueError, "BuildCheck %r identified error" % self.name
        else:
            # Indexed component
            for index in self._index:
                res = apply_indexed_rule(self, self._rule, self._parent(), index)
                if not res:
                    raise ValueError, "BuildCheck %r identified error with index %r" % (self.name, str(index))



"""
       / 2x          , 0 <= x <= 1
       | (1/2)x+(3/2), 1 <= x <= 3
f(x) = | -3x+12      , 3 <= x <= 5
       \ 2x-13       , 5 <= x <= 6
"""

from coopr.pyomo import *

INDEX_SET1 = ['1','2','3','40'] # There will be two copies of this function
INDEX_SET2 = [(t1,t2) for t1 in range(1,4) for t2 in range(1,5)]
DOMAIN_PTS = dict([((t1,t2,t3),[float(i) for i in [0,1,3,5,6]]) for t1 in INDEX_SET1 for (t2,t3) in INDEX_SET2])
RANGE_PTS = {0.0:0.0, 1.0:2.0, 3.0:3.0, 5.0:-3.0, 6.0:-1.0}

def F(model,t1,t2,t3,x):
    return RANGE_PTS[x]*model.p[t1,t2,t3]

def define_model(**kwds):

    model = ConcreteModel()

    model.x = Var(INDEX_SET1, INDEX_SET2, bounds=(0,6)) # domain variable
    model.Fx = Var(INDEX_SET1, INDEX_SET2) # range variable
    model.p = Param(INDEX_SET1, INDEX_SET2, initialize=1.0)

    model.obj = Objective(expr=summation(model.Fx), sense=kwds.pop('sense',maximize))

    model.piecewise = Piecewise(INDEX_SET1,INDEX_SET2,model.Fx,model.x,
                                  pw_pts=DOMAIN_PTS,
                                  f_rule=F, **kwds)

    #Fix the answer for testing purposes
    model.set_answer_constraint1 = Constraint(INDEX_SET2,expr= lambda model,t2,t3: model.x['1',t2,t3] == 0.0)
    model.set_answer_constraint2 = Constraint(INDEX_SET2,expr= lambda model,t2,t3: model.x['2',t2,t3] == 3.0)
    model.set_answer_constraint3 = Constraint(INDEX_SET2,expr= lambda model,t2,t3: model.x['3',t2,t3] == 5.5)
    model.set_answer_constraint4 = Constraint(INDEX_SET2,expr= lambda model,t2,t3: model.x['40',t2,t3] == 6.0)

    return model



from pyutilib.component.core import *
from base import *

import ast

class ModelTrackerHook(Plugin):
    implements(IPreCheckHook)

    def precheck(self, runner, script, info):
        # create models dict if nonexistent
        if getattr(script, 'modelVars', None) is None:
            script.modelVars = {}

        # parse AST node
        if isinstance(info, ast.Assign):
            if isinstance(info.value, ast.Call):
                if isinstance(info.value.func, ast.Name):
                    if info.value.func.id.endswith("Model"):
                        for target in info.targets:
                            if isinstance(target, ast.Name):
                                script.modelVars[target.id] = info.value.func.id
                            elif isinstance(target, ast.Tuple):
                                for elt in target.elts:
                                    if isinstance(elt, ast.Name):
                                        script.modelVars[elt.id] = info.value.func.id
                    else:
                        for target in info.targets:
                            if isinstance(target, ast.Name):
                                if target.id in script.modelVars:
                                    del script.modelVars[target.id]
                            elif isinstance(target, ast.Tuple):
                                for elt in target.elts:
                                    if isinstance(elt, ast.Name):
                                        if elt.id in script.modelVars:
                                            del script.modelVars[elt.id]



from coopr.pyomo import *
from pyutilib.component.core import implements, Plugin, Interface
from coopr.pyomo.transform import Transformation

__all__ = ["IsomorphicTransformation"]

class IsomorphicTransformation(Transformation):
    """
    Base class for 'lossless' transformations for which a bijective
    mapping between optimal variable values and the optimal cost
    exists.
    """

    def __init__(self, **kwds):
        kwds["name"] = kwds.get("name", "isomorphic_transformation")
        super(IsomorphicTransformation, self).__init__(**kwds)



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Objective', 'simple_objective_rule', '_ObjectiveData', 'minimize', 'maximize']

import sys
import logging
import weakref

import pyutilib.math
import pyutilib.misc
from pyutilib.component.core import alias

from numvalue import NumericValue, as_numeric, value, native_numeric_types
from expr import Expression
from indexed_component import IndexedComponent
from misc import create_name, apply_indexed_rule
from sets import Set
from var import _VarData
from set_types import Reals

#import time
#from coopr.pyomo.base.expr import generate_expression, generate_relational_expression
#from coopr.pyomo.base.intrinsic_functions import generate_intrinsic_function_expression

logger = logging.getLogger('coopr.pyomo')

minimize=1
maximize=-1


def simple_objective_rule( fn ):
    """
    This is a decorator that translates None into Constraint.Skip.
    This supports a simpler syntax in objective rules, though these
    can be more difficult to debug when errors occur.

    Example use:

    @simple_objective_rule
    def O_rule(model, i, j):
        ...
    """

    def wrapper_function ( *args, **kwargs ):
        value = fn( *args, **kwargs )
        if value is None:
            return Objective.Skip
        return value
    return wrapper_function


class _ObjectiveData(NumericValue):
    """
    This class defines the data for a single objective.

    Note that this is a subclass of NumericValue to allow
    objectives to be used as part of expressions.

    Constructor arguments:
        expr            The expression for this objective.
        name            The name of this objective.
        component       The Objective object that owns this data.

    Public class attributes:
        active          A boolean that is true if this objective is active in the model.
        component       The objective component.
        domain          A domain object that restricts possible values for this objective.
        expr            The Pyomo expression for this objective
        id              An integer value that defines 
        index           The index value for this objective
        name            The name of this objective
        value           The numeric value of this objective

    Private class attributes:
        _repn           TBD
    """

    __slots__ = ['component', 'expr', 'id', 'active', 'repn', 'ampl_repn', 'index']

    def __init__(self, expr, name, component):
        # The following three lines are equivalent to calling the
        # basic NumericValue constructor, i.e., as follows:
        # NumericValue.__init__(self, name, Reals, None)
        self.name = name
        self.domain = Reals
        self.value = None
        #
        self.active = True
        self.ampl_repn = None
        self.component = weakref.ref(component)
        self.expr = expr
        self.id = -1
        self.index = None
        self.repn = None

    def __getstate__(self):
        """
        This method is required because this class uses slots.
        """
        result = NumericValue.__getstate__(self)
        for i in _ObjectiveData.__slots__:
            result[i] = getattr(self, i)
        if type(result['component']) is weakref.ref:
            result['component'] = result['component']()
        return result

    def __setstate__(self, state):
        """
        This method is required because this class uses slots.
        """
        for (slot_name, value) in state.iteritems():
            setattr(self, slot_name, value)
        self.component = weakref.ref(self.component)

    def __call__(self, exception=True):
        """
        Compute the value of this objective.

        This method does not simply return self.value because that data value may be out of date
        w.r.t. the value of decision variables.
        """
        if self.expr is None:
            return None
        return self.expr()

    def polynomial_degree(self):
        """
        Return the polynomial degree of the objective expression.
        """
        if self.expr is None:
            return None
        return self.expr.polynomial_degree()

    def activate(self):
        """
        Activate this objective.
        """
        self.active=True

    def deactivate(self):
        """
        Deactivate this objective.
        """
        self.active=False


class Objective(IndexedComponent, NumericValue):
    """
    This modeling component defines an objective expression.

    Note that this is a subclass of NumericValue to allow
    objectives to be used as part of expressions.

    Constructor arguments:
        expr            A Pyomo expression for this objective
        doc             A text string describing this component
        name            A name for this component
        noruleinit      Indicate that its OK that no initialization is
                            specified
        rule            A function that is used to construct objective
                            expressions
        sense           Indicate whether minimizing or maximizing
        
    Public class attributes:
        active          A boolean that is true if this component will be 
                            used to construct a model instance
        doc             A text string describing this component
        domain          A domain object that restricts possible values for this object.
        expr*           Not defined
        id*             Not defined
        index*          Not defined
        name            A name for this object.
        rule            The rule used to initialize the objective(s)
        sense           The objective sense
        trivial         A boolean if all objective indices have trivial expressions
        value           The numeric value of this object.

    Private class attributes:
        _constructed    A boolean that is true if this component has been
                            constructed
        _data           A dictionary from the index set to component data objects
        _index          The set of valid indices
        _index_set      A tuple of set objects that represents the index set
        _model          A weakref to the model that owns this component
        _ndim           The dimension of the index set
        _no_rule_init   A boolean that indicates if an initialization rule is needed
        _parent         A weakref to the parent block that owns this component
        _type           The class type for the derived subclass
    """

    alias('Objective', doc='Expressions that are minimized or maximized in a model.', subclass=True)

    Skip        = (1000,)
    NoObjective = (1000,)

    def __new__(cls, *args, **kwds):
        if cls != Objective:
            return super(Objective, cls).__new__(cls)
        if args == ():
            return _ObjectiveElement.__new__(_ObjectiveElement)
        else:
            return _ObjectiveArray.__new__(_ObjectiveArray)

    def __init__(self, *args, **kwargs):
        # Initialize values defined in NumericValue base class
        self.domain = None
        self.value = None
        #
        self.sense = kwargs.pop('sense', minimize )
        tmprule  = kwargs.pop('rule', None )
        self.rule  = kwargs.pop('expr', tmprule )
        self._no_rule_init = kwargs.pop('noruleinit', None )
        self.trivial = False
        self._constructed = False
        #
        kwargs.setdefault('ctype', Objective)
        IndexedComponent.__init__(self, *args, **kwargs)

    def __getstate__(self):
        """
        This method is required because this class uses slots.  Call base class __getstate__
        methods since this class does not define additional slots.
        """
        numvalue_result = NumericValue.__getstate__(self)
        indexed_component_result = IndexedComponent.__getstate__(self)
        return dict(numvalue_result.items() + indexed_component_result.items())

    def __setstate__(self, state):
        """
        This method is required because this class uses slots.  Call base class __setstate__
        methods since this class does not define additional slots.
        """
        NumericValue.__setstate__(self, state)
        IndexedComponent.__setstate__(self, state)
 
    def is_minimizing ( self ):
        """Return true if this is a minimization objective"""
        return self.sense == minimize

    def construct(self, data=None):
        """Construct the expression(s) for this objective."""
        if __debug__:
            logger.debug("Constructing objective %s", self.name)
        if (self._no_rule_init is not None) and (self.rule is not None):
            msg = 'WARNING: noruleinit keyword is being used in conjunction ' \
                  "with rule keyword for objective '%s'; defaulting to "      \
                  'rule-based construction'
            print msg % self.name
        if self.rule is None:
            if self._no_rule_init is None:
                msg = 'WARNING: No construction rule or expression specified ' \
                      "for objective '%s'"
                print msg % self.name
            return
        if self._constructed:
            return
        self._constructed=True
        # NumbericValue -> single Var, single Param, Expression
        if isinstance(self.rule,NumericValue):
            if None in self._index or len(self._index) == 0 or self._index[None] is None:
                self._data[None].expr = self.rule
            else:
                msg = 'Cannot define multiple indices in an objective with a' \
                      'single expression'
                raise IndexError, msg
        # A very uncommon case where the objective is simply a number
        # Number -> int, long, float
        elif self.rule.__class__ in native_numeric_types:
            if None in self._index or len(self._index) == 0 or self._index[None] is None:
                self._data[None].expr = as_numeric(self.rule)
            else:
                msg = 'Cannot define multiple indices in an objective with a' \
                      'single expression'
                raise IndexError, msg
        #
        elif self.rule is not None:
            if self._ndim==0:
                tmp = self.rule(self._parent())
                if tmp is None:
                    raise ValueError, "Objective rule returned None instead of Objective.Skip"
                if not (tmp.__class__ is tuple and tmp == Objective.Skip):
                    self._data[None].expr = as_numeric(tmp)
            else:
                _name=self.name
                for val in self._index:
                    tmp = apply_indexed_rule(self, self.rule, self._parent(), val)
                    if tmp is None:
                        raise ValueError, "Objective rule returned None instead of Objective.Skip"
                    if not (tmp.__class__ is tuple and tmp == Objective.Skip):
                        self._data[val] = _ObjectiveData(as_numeric(tmp), create_name(_name,val), self)
                        self._data[val].index = val 
        #
        #if None in self._data:
            #self._data[None].name = self.name
        #print "[%.1f] %s: %d,%d,%d" % \
        #      (time.time(), self.name,
        #       generate_expression.clone_counter,
        #       generate_relational_expression.clone_counter,
        #       generate_intrinsic_function_expression.clone_counter)


    def pprint(self, ostream=None, verbose=False):
        """Print the objective value(s)"""
        if ostream is None:
            ostream = sys.stdout
        print >>ostream, "  ",self.name,":",
        if not self.doc is None:
            print >>ostream, self.doc
            print >>ostream, "  ",
        print >>ostream, "\tSize="+str(len(self._data.keys())),
        if isinstance(self._index, Set):
            print >>ostream, "\tIndex=",self._index.name
        else:
            print >>ostream,""
        for key in self._data:
            if not self._data[key].expr is None:
                print >>ostream, "\t",
                self._data[key].expr.pprint(ostream, verbose=verbose)
                print >>ostream, ""

    def display(self, prefix="", ostream=None):
        """Provide a verbose display of this object"""
        if ostream is None:
            ostream = sys.stdout
        print >>ostream, prefix+"Objective "+self.name,":",
        print >>ostream, "  Size="+str(len(self))
        if None in self._data:
            if self._data[None].expr is None:
                val = 'none'
            else:
                val = pyutilib.misc.format_io(self._data[None].expr(exception=False))
            print >>ostream, '%s  Value=%s' % (prefix, val)
        else:
            for key in self._data:
                if not self._data[key].active:
                    continue
                val = self._data[key].expr(exception=False)
                print >>ostream, prefix+"  "+str(key)+" : "+str(val)


class _ObjectiveElement(Objective, _ObjectiveData):

    def __init__(self, *args, **kwd):
        _ObjectiveData.__init__(self, None, kwd.get('name', None), self)
        Objective.__init__(self, *args, **kwd)
        self._data[None] = self

    def __call__(self, exception=True):
        """Compute the value of the objective expression"""
        if self.expr is None:
            return None
        return value(self.expr)

    def __getstate__(self):
        #
        # picks up all slots in the _ObjectiveData class, and in particular
        # makes sure the "component" value is not a weakref,
        # but the actual objects themselves.
        #
        result1 = _ObjectiveData.__getstate__(self)
        result1['component'] = self
        result2 = Objective.__getstate__(self)
        return dict(result1.items() + result2.items())

    def __setstate__(self, state):
        for (slot_name, value) in state.iteritems():
            setattr(self, slot_name, value)

        self.component = weakref.ref(self)
        # this is hackish, in that it replicates code from the base Component setstate method - 
        # revisit at   some point.
        if '_parent' in self.__dict__.keys() and self._parent is not None and type(self._parent) != weakref.ref:
            self._parent = weakref.ref(self._parent)


class _ObjectiveArray(Objective):

    def __call__(self, exception=True):
        """Compute the value of the objective body"""
        if exception:
            msg = 'Cannot compute the value of an array of objectives'
            raise ValueError, msg




"""
       / -9x-20, -5 <= x <= -4
       | -7x-12, -4 <= x <= -3
       | -5x-6 , -3 <= x <= -2
       | -3x-2 , -2 <= x <= -1
f(x) = | 1     , -1 <= x <=  1 
       | 3x-2  ,  1 <= x <=  2
       | 5x-6  ,  2 <= x <=  3
       \ 7x-12 ,  3 <= x <=  4

"""

from coopr.pyomo import *

INDEX_SET1 = range(1,8)
INDEX_SET2 = range(0,2)
DOMAIN_PTS = dict([((t1,t2),[float(i) for i in (range(-5,0)+range(1,5))]) for t1 in INDEX_SET1 for t2 in INDEX_SET2])

 
def F(model,t1,t2,x):
    return (x**2)*model.p[t1,t2]

def define_model(**kwds):

    model = ConcreteModel()

    model.x = Var(INDEX_SET1, INDEX_SET2, bounds=(-5,4)) # domain variable
    model.Fx = Var(INDEX_SET1, INDEX_SET2) # range variable
    model.p = Param(INDEX_SET1, INDEX_SET2, initialize=1.0)

    model.obj = Objective(expr=summation(model.Fx), sense=kwds.pop('sense',maximize))

    model.piecewise = Piecewise(INDEX_SET1,INDEX_SET2,model.Fx,model.x,
                                  pw_pts=DOMAIN_PTS,
                                  f_rule=F, **kwds)

    #Fix the answer for testing purposes
    model.set_answer_constraint1 = Constraint(expr= model.x[1,0] == -5.0)
    model.set_answer_constraint2 = Constraint(expr= model.x[2,0] == -3.0)
    model.set_answer_constraint3 = Constraint(expr= model.x[3,0] == -2.5)
    model.set_answer_constraint4 = Constraint(expr= model.x[4,0] == -1.5)
    model.set_answer_constraint5 = Constraint(expr= model.x[5,0] == 2.0)
    model.set_answer_constraint6 = Constraint(expr= model.x[6,0] == 3.5)
    model.set_answer_constraint7 = Constraint(expr= model.x[7,0] == 4.0)
    model.set_answer_constraint8 = Constraint(expr= model.x[1,1] == -5.0)
    model.set_answer_constraint9 = Constraint(expr= model.x[2,1] == -3.0)
    model.set_answer_constraint10 = Constraint(expr= model.x[3,1] == -2.5)
    model.set_answer_constraint11 = Constraint(expr= model.x[4,1] == -1.5)
    model.set_answer_constraint12 = Constraint(expr= model.x[5,1] == 2.0)
    model.set_answer_constraint13 = Constraint(expr= model.x[6,1] == 3.5)
    model.set_answer_constraint14 = Constraint(expr= model.x[7,1] == 4.0)    
    
    return model



#
# Unit Tests for Python numeric values
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
currdir = dirname(abspath(__file__))+os.sep

import pyutilib.th as unittest
from coopr.pyomo import *
from nose.tools import nottest

class Test_value(unittest.TestCase):

    def test_none(self):
        val = None
        self.assertEquals(val, value(val))

    def test_bool(self):
        val = False
        self.assertEquals(val, value(val))

    def test_float(self):
        val = 1.1
        self.assertEquals(val, value(val))

    def test_int(self):
        val = 1
        self.assertEquals(val, value(val))

    def test_long(self):
        val = long(1e10)
        self.assertEquals(val, value(val))

    def test_nan(self):
        val = pyutilib.math.nan
        self.assertEquals(id(val), id(value(val)))

    def test_inf(self):
        val = pyutilib.math.infinity
        self.assertEquals(id(val), id(value(val)))

    def test_string(self):
        val = 'foo'
        self.assertEquals(val, value(val))

    def test_const1(self):
        val = NumericConstant(None, None, 1.0)
        self.assertEquals(1.0, value(val))

    def test_const2(self):
        val = NumericConstant(None, Any, 'foo')
        self.assertEquals('foo', value(val))

    def test_const3(self):
        val = NumericConstant(None, None, pyutilib.math.nan)
        self.assertEquals(id(pyutilib.math.nan), id(value(val)))

    def test_const4(self):
        val = NumericConstant(None, None, pyutilib.math.infinity)
        self.assertEquals(id(pyutilib.math.infinity), id(value(val)))

    def test_val1(self):
        val = NumericValue(None, None, 1.0)
        self.assertEquals(1.0, value(val))

    def test_val2(self):
        val = NumericValue(None, Any, 'foo')
        self.assertEquals('foo', value(val))

    def test_val3(self):
        val = NumericValue(None, None, pyutilib.math.nan)
        self.assertEquals(id(pyutilib.math.nan), id(value(val)))

    def test_val4(self):
        val = NumericValue(None, None, pyutilib.math.infinity)
        self.assertEquals(id(pyutilib.math.infinity), id(value(val)))

    def test_error1(self):
        class A(object): pass
        val = A()
        try:
            value(val)
            self.fail("Expected ValueError")
        except ValueError:
            pass

    def test_error2(self):
        val = NumericConstant(None, Any, None)
        try:
            value(val)
            self.fail("Expected ValueError")
        except ValueError:
            pass


class Test_is_constant(unittest.TestCase):

    def test_none(self):
        self.assertTrue(is_constant(None))

    def test_bool(self):
        self.assertTrue(is_constant(True))

    def test_float(self):
        self.assertTrue(is_constant(1.1))

    def test_int(self):
        self.assertTrue(is_constant(1))

    def test_long(self):
        val = long(1e10)
        self.assertTrue(is_constant(val))

    def test_string(self):
        self.assertTrue(is_constant('foo'))

    def test_const1(self):
        val = NumericConstant(None, None, 1.0)
        self.assertTrue(is_constant(val))

    def test_const2(self):
        val = NumericConstant(None, Any, 'foo')
        self.assertTrue(is_constant(val))

    def test_error(self):
        class A(object): pass
        val = A()
        try:
            is_constant(val)
            self.fail("Expected ValueError")
        except ValueError:
            pass


class Test_as_numeric(unittest.TestCase):

    def test_none(self):
        val = None
        try:
            as_numeric(val)
            self.fail("Expected ValueError")
        except:
            pass

    def test_bool(self):
        val = False
        try:
            as_numeric(val)
            self.fail("Expected ValueError")
        except:
            pass
        val = True
        try:
            as_numeric(val)
            self.fail("Expected ValueError")
        except:
            pass

    def test_float(self):
        val = 1.1
        self.assertEquals(val, as_numeric(val))

    def test_int(self):
        val = 1
        self.assertEquals(val, as_numeric(val))

    def test_long(self):
        val = long(1e10)
        self.assertEquals(val, as_numeric(val))

    def test_string(self):
        val = 'foo'
        try:
            as_numeric(val)
            self.fail("Expected ValueError")
        except:
            pass

    def test_const1(self):
        val = NumericConstant(None, None, 1.0)
        self.assertEquals(1.0, as_numeric(val))

    def test_const2(self):
        val = NumericConstant(None, Any, 'foo')
        try:
            as_numeric(val)
            self.fail("Expected ValueError")
        except:
            pass

    def test_error1(self):
        class A(object): pass
        val = A()
        try:
            as_numeric(val)
            self.fail("Expected ValueError")
        except ValueError:
            pass

    def test_error2(self):
        val = NumericConstant(None, Any, None)
        num = as_numeric(val)
        try:
            value(num)
            self.fail("Expected ValueError")
        except ValueError:
            pass


if __name__ == "__main__":
    unittest.main()




#
# Author:  Gabe Hackebeil
# Purpose: For regression testing to ensure that the Pyomo
#          NL writer properly modifies product expressions 
#          with only constant terms in the denominator (that
#          are involved in nonlinear expressions).
#          The ASL differentiation routines seem to have a 
#          bug that causes the lagrangian hessian to become
#          dense unless this constant term in moved to the 
#          numerator. 
#
#          This test model relies on the asl_test executable. It
#          will not solve if sent to a real optimizer.
#

from coopr.pyomo import *

model = ConcreteModel()
model.x = Var(bounds=(-1.0,1.0),initialize=1.0)
model.y = Var(bounds=(-1.0,1.0),initialize=2.0)
model.v = Var(bounds=(-1.0,1.0),initialize=3.0)
model.p = Var(initialize=2.0)
model.p.fixed = True
model.q = Param(initialize=2.0)

model.OBJ = Objective(expr=model.x)
model.CON1a = Constraint(expr=1.0/model.p/model.q*model.v*(model.x-model.y) == 2.0)
model.CON2a = Constraint(expr=model.v*1.0/model.p/model.p*(model.x-model.y) == 2.0)
model.CON3a = Constraint(expr=model.v*(model.x-model.y)/model.p/model.q == 2.0)
model.CON4a = Constraint(expr=model.v*(model.x/model.p/model.q-model.y/model.p/model.q) == 2.0)
model.CON5a = Constraint(expr=model.v*(model.x-model.y)*(1.0/model.p/model.q) == 2.0)
model.CON6a = Constraint(expr=model.v*(model.x-model.y) == 2.0*model.p*model.q)

model.CON1b = Constraint(expr=1.0/(model.p*model.q)*model.v*(model.x-model.y) == 2.0)
model.CON2b = Constraint(expr=model.v*1.0/(model.p*model.p)*(model.x-model.y) == 2.0)
model.CON3b = Constraint(expr=model.v*(model.x-model.y)/(model.p*model.q) == 2.0)
model.CON4b = Constraint(expr=model.v*(model.x/(model.p*model.q)-model.y/(model.p*model.q)) == 2.0)
model.CON5b = Constraint(expr=model.v*(model.x-model.y)*(1.0/(model.p*model.q)) == 2.0)
model.CON6b = Constraint(expr=model.v*(model.x-model.y) == 2.0*(model.p*model.q))

model.CON1c = Constraint(expr=1.0/(model.p+model.q)*model.v*(model.x-model.y) == 2.0)
model.CON2c = Constraint(expr=model.v*1.0/(model.p+model.p)*(model.x-model.y) == 2.0)
model.CON3c = Constraint(expr=model.v*(model.x-model.y)/(model.p+model.q) == 2.0)
model.CON4c = Constraint(expr=model.v*(model.x/(model.p+model.q)-model.y/(model.p+model.q)) == 2.0)
model.CON5c = Constraint(expr=model.v*(model.x-model.y)*(1.0/(model.p+model.q)) == 2.0)
model.CON6c = Constraint(expr=model.v*(model.x-model.y) == 2.0*(model.p+model.q))

model.CON1d = Constraint(expr=1.0/((model.p+model.q)**2)*model.v*(model.x-model.y) == 2.0)
model.CON2d = Constraint(expr=model.v*1.0/((model.p+model.p)**2)*(model.x-model.y) == 2.0)
model.CON3d = Constraint(expr=model.v*(model.x-model.y)/((model.p+model.q)**2) == 2.0)
model.CON4d = Constraint(expr=model.v*(model.x/((model.p+model.q)**2)-model.y/((model.p+model.q)**2)) == 2.0)
model.CON5d = Constraint(expr=model.v*(model.x-model.y)*(1.0/((model.p+model.q)**2)) == 2.0)
model.CON6d = Constraint(expr=model.v*(model.x-model.y) == 2.0*((model.p+model.q)**2))




#
# Example 2.1 - Allen Holder
#

from coopr.pyomo import *

# Instantiate the model
model = AbstractModel()

# Sets
model.DoorType = Set()
model.MachineType = Set()
model.MarketDoorType1 = Set(within=model.DoorType)
model.MarketDoorType2 = Set(within=model.DoorType)

# Parameters
model.Hours = Param(model.DoorType, model.MachineType, within=NonNegativeReals)
model.Labor = Param(model.DoorType, model.MachineType, within=NonNegativeReals)
model.Profit = Param(model.DoorType, within=NonNegativeReals)
model.MachineLimit = Param(model.MachineType, within=NonNegativeReals)
model.LaborLimit = Param(within=NonNegativeReals)

# Variables
model.NumDoors = Var(model.DoorType, within=NonNegativeIntegers)

# Objective
def CalcProfit(M):
    return sum (M.NumDoors[d]*M.Profit[d] for d in M.DoorType)
model.TotProf = Objective(rule=CalcProfit, sense=maximize)

# Constraints
def EnsureMachineLimit(M, m):
    return sum (M.NumDoors[d]*M.Labor[d,m] for d in M.DoorType) \
           <= M.MachineLimit[m]
model.MachineUpBound = Constraint(model.MachineType, rule=EnsureMachineLimit)

def EnsureLaborLimit(M):
    return sum (M.NumDoors[d]*M.Labor[d,m] \
                for d in M.DoorType for m in M.MachineType) \
           <= M.LaborLimit
model.MachineUpBound = Constraint(rule=EnsureLaborLimit)

def EnsureMarketRatio(M):
    return sum (M.NumDoors[d] for d in M.MarketDoorType1) \
           <= sum (M.NumDoors[d] for d in M.MarketDoorType2)
model.MarketRatio = Constraint(rule=EnsureMarketRatio)



"""
       / 2x          , 0 <= x <= 1
       | (1/2)x+(3/2), 1 <= x <= 3
f(x) = | -3x+12      , 3 <= x <= 5
       \ 2x-13       , 5 <= x <= 6
"""

from coopr.pyomo import *

INDEX_SET = [1,2,3,4] # There will be two copies of this function
DOMAIN_PTS = dict([(t,[float(i) for i in [0,1,3,5,6]]) for t in INDEX_SET])
RANGE_PTS = {0.0:0.0, 1.0:2.0, 3.0:3.0, 5.0:-3.0, 6.0:-1.0}

def F(model,t,x):
    return RANGE_PTS[x]*model.p[t]

def define_model(**kwds):

    model = ConcreteModel()

    model.x = Var(INDEX_SET, bounds=(0,6)) # domain variable
    model.Fx = Var(INDEX_SET) # range variable
    model.p = Var(INDEX_SET, initialize=1.0)

    model.obj = Objective(expr=summation(model.Fx), sense=kwds.pop('sense',maximize))

    model.piecewise = Piecewise(INDEX_SET,model.Fx,model.x,
                                  pw_pts=DOMAIN_PTS,
                                  f_rule=F, **kwds)

    #Fix the answer for testing purposes
    model.set_answer_constraint1 = Constraint(expr= model.x[1] == 0.0)
    model.set_answer_constraint2 = Constraint(expr= model.x[2] == 3.0)
    model.set_answer_constraint3 = Constraint(expr= model.x[3] == 5.5)
    model.set_answer_constraint4 = Constraint(expr= model.x[4] == 6.0)
    
    return model



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2012 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['instance2dat']

from coopr.pyomo.base import Set, Param, value

import types

# IMPT: Only works on non-nested block models at the moment!

def instance2dat(instance, output_filename):

    output_file = open(output_filename,"w")

    for set_name, set_object in instance.active_components(Set).iteritems():
        if (set_object.initialize is not None) and (type(set_object.initialize) is types.FunctionType):
            continue

        if (set_name.find("_index") == -1) and (set_name.find("_domain") == -1):
            if set_object.dim() == 0:
                if len(set_object) == 0:
                    continue
                print >>output_file, "set "+set_name+" := "
                for element in set_object:
                    print >>output_file, element
                print >>output_file, ";"
            elif set_object.dim() == 1:
                for index in set_object:
                    print >>output_file, "set "+set_name+"[\""+str(index)+"\"]"+" := ",
                    for element in set_object[index]:
                        print >>output_file, element,
                    print >>output_file, ";"
            else:
                print >>output_file, "***MULTIPLY INDEXED SETS NOT IMPLEMENTED!!!"
                pass

            print >>output_file, ""

    for param_name, param_object in instance.active_components(Param).iteritems():
        if (param_object._initialize is not None) and (type(param_object._initialize) is types.FunctionType):
            continue
        elif len(param_object) == 0:
            continue

        if None in param_object:
            print >>output_file, "param "+param_name+" := "+str(value(param_object[None]))+" ;"
            print >>output_file, ""
        else:
            print >>output_file, "param "+param_name+" := "
            if param_object.dim() == 1:
                for index in param_object:
                    print >>output_file, index, str(value(param_object[index]))
            else:
                for index in param_object:
                    for i in index:
                        print >>output_file, i,
                    print >>output_file, str(value(param_object[index]))
            print >>output_file, ";"
            print >>output_file, ""

    output_file.close()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

import pyutilib.misc
import pyutilib.component.core
import coopr.core
from coopr.pyomo.base import IPyomoPresolveAction


@coopr.core.coopr_api(namespace='pyomo.model')
def simple_preprocessor(data, model=None):
    """
    This plugin simply applies preprocess actions in a fixed order.

    Required:
        model:      A concrete model instance.
    """
    coopr.core.CooprAPIFactory('pyomo.model.compute_canonical_repn')(data, model=model)  
    #
    # Process the presolver actions
    #
    actions = pyutilib.component.core.ExtensionPoint(IPyomoPresolveAction)
    active_actions = set()
    action_rank = {}
    #
    # Collect active actions
    #
    for action in actions():
        tmp = actions.service(action)
        if tmp is None:
            raise ValueError, "Cannot activate unknown action %s" % action
        active_actions.add(action)
        if rank is None:
            rank = tmp.rank()
        action_rank[action] = rank
    #
    # Sort active actions
    #
    actions = list(active_actions)
    ranks = []
    for item in actions:
        ranks.append(action_rank[item])
    index = pyutilib.misc.sort_index(ranks)
    sorted=[]
    for i in index:
        sorted.append( actions[i] )
    #
    # Preprocess active actions in order
    #
    for action in sorted:
        model = actions.service(action).preprocess(model)




import os

import pyutilib.th as unittest
from pyutilib.misc.pyyaml_util import *
import pyutilib.common
import coopr.pyomo.scripting.pyomo as pyomo
import coopr.plugins
from nose.tools import nottest

currdir = os.path.dirname(os.path.abspath(__file__))
exdir = os.path.abspath(os.path.join(currdir, '..', '..', '..', '..', '..', 'examples', 'pyomo', 'diet'))

sqlite3_available = pyodbc_available = False
try:
    import sqlite3
    sqlite3_available = True
except ImportError:
    pass
try:
    import pyodbc
    pyodbc_available = True
except ImportError:
    pass

def glpk_available():
    try:
        glpk = coopr.plugins.solvers.GLPK(keepFiles=False)
        return (not glpk.executable() is None) and glpk.available(False)
    except pyutilib.common.ApplicationError:
        return False

@unittest.skipUnless(glpk_available(), "GLPK is not installed")
class Test(unittest.TestCase):

    @nottest
    def pyomo(self, *args, **kwargs):
        """
        Run Pyomo with the given arguments. `args` should be a list with
        one argument token per string item.
        """

        if isinstance(args, str):
            args = args.split()
        args = list(map(str, args))

        outputpath = kwargs.pop('outputpath', os.path.join(exdir, 'results.jsn'))
        args = ['--json', '-c', '--quiet', '--save-results', outputpath] + args
        print "Args:", ' '.join(args)

        old_path = os.getcwd()
        os.chdir(exdir)
        output = pyomo.run(args)
        os.chdir(old_path)

        return outputpath

    def test_pyomo_dat(self):
        results_file = self.pyomo(os.path.join(exdir, 'diet1.py'), os.path.join(exdir, 'diet.dat'), outputpath=os.path.join(currdir, 'pyomo_dat.jsn'))
        baseline_file = os.path.join(currdir, 'baselines', 'diet1_pyomo_dat.jsn')
        self.assertMatchesJsonBaseline(results_file, baseline_file)

    @unittest.skipUnless(pyodbc_available, "Requires PyODBC")
    def test_pyomo_mdb(self):
        results_file = self.pyomo(os.path.join(exdir, 'diet1.py'), os.path.join(exdir, 'diet1.db.dat'), outputpath=os.path.join(currdir, 'pyomo_mdb.jsn'))
        baseline_file = os.path.join(currdir, 'baselines', 'diet1_pyomo_mdb.jsn')
        self.assertMatchesJsonBaseline(results_file, baseline_file)

    @unittest.skipUnless(pyodbc_available, "Requires PyODBC")
    def test_mdb_equality(self):
        dat_results_file = self.pyomo(os.path.join(exdir, 'diet1.py'), os.path.join(exdir, 'diet.dat'), outputpath=os.path.join(currdir, 'dat_results.jsn'))
        mdb_results_file = self.pyomo(os.path.join(exdir, 'diet1.py'), os.path.join(exdir, 'diet1.db.dat'), outputpath=os.path.join(currdir, 'mdb_results.jsn'))
        self.assertMatchesJsonBaseline(dat_results_file, mdb_results_file)
        os.remove(mdb_results_file)

    @unittest.skipUnless(sqlite3_available, "Requires SQLite3")
    def test_pyomo_sqlite3(self):
        results_file = self.pyomo(os.path.join(exdir, 'diet1.py'), os.path.join(exdir, 'diet1.sqlite.dat'), outputpath=os.path.join(currdir, 'pyomo_sqlite3.jsn'))
        baseline_file = os.path.join(currdir, 'baselines', 'diet1_pyomo_sqlite3.jsn')
        self.assertMatchesJsonBaseline(results_file, baseline_file)

    @unittest.skipUnless(sqlite3_available, "Requires SQLite3")
    def test_sqlite_equality(self):
        dat_results_file = self.pyomo(os.path.join(exdir, 'diet1.py'), os.path.join(exdir, 'diet.dat'), outputpath=os.path.join(currdir, 'dat_results.jsn'))
        sqlite_results_file = self.pyomo(os.path.join(exdir, 'diet1.py'), os.path.join(exdir, 'diet1.sqlite.dat'), outputpath=os.path.join(currdir, 'sqlite_results.jsn'))
        self.assertMatchesJsonBaseline(dat_results_file, sqlite_results_file)
        os.remove(sqlite_results_file)

if __name__ == "__main__":
    unittest.main()




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = [ 'value', 'NumericValue', 'as_numeric', 'NumericConstant',
            'is_constant', 'is_fixed']

import sys
import logging

import pyutilib.math
from set_types import Reals, Any

logger = logging.getLogger('coopr.pyomo')

def create_name(name, ndx):
    """
    Create a canonical name for a component using the given index.
    """
    if ndx is None:
        return name
    if type(ndx) is tuple:
        tmp = str(ndx).replace(', ',',')
        return name+"["+tmp[1:-1]+"]"
    return name+"["+str(ndx)+"]"


##------------------------------------------------------------------------
##
## Standard types of expressions
##
##------------------------------------------------------------------------


def _old_value(obj):
    """
    A utility function that returns the value of a Pyomo object or expression.

    If the argument is None, a numeric value or a string, then this
    function simply returns the argument.  Otherwise, if the argument is
    a NumericValue then the __call__ method is executed.
    """
    if obj is None:
        return None
    if type(obj) in (bool,int,long,float,str):
        return obj
    if not isinstance(obj, NumericValue):
        raise ValueError("Object %s is not a NumericValue object" % (obj,))
    tmp = obj()
    if tmp is None:
        raise ValueError("No value for uninitialized NumericValue object %s"
                         % (obj.name,))
    return tmp

# It is *significantly* faster to build the list of types we want to
# test against as a "static" set, and not to regenerate it locally for
# every call.  Plus, this allows us to dynamically augment the set with
# new "native" types (e.g., from NumPy)
native_numeric_types = set([ int, float, long ])
native_types = set([ bool, str, type(None) ]).union( native_numeric_types )

def value(obj):
    """
    A utility function that returns the value of a Pyomo object or expression.

    If the argument is None, a numeric value or a string, then this
    function simply returns the argument.  Otherwise, if the argument is
    a NumericValue then the __call__ method is executed.
    """
    if obj.__class__ in native_types:
        return obj
    try:
        numeric = obj.as_numeric()
    except AttributeError:
        numeric = as_numeric(obj)
    try:
        tmp = numeric()
    except:
        logger.error(
            "evaluating object as numeric value: %s\n\t(object: %s)\n\t%s"
            % (obj, type(obj), sys.exc_info()[1]))
        raise ValueError
    
    if tmp is None:
        raise ValueError("No value for uninitialized NumericValue object %s"
                         % (obj.name,))
    return tmp


def is_constant(obj):
    """
    A utility function that returns a boolean that indicates whether the
    object is a constant
    """
    # This method is rarely, if ever, called.  Plus, since the
    # expression generation (and constraint generation) system converts
    # everything to NumericValues, it is better (i.e., faster) to assume
    # that the obj is a NumericValue
    #
    # JDS: NB: I am not sure why we allow str to be a constant, but
    # since we have historically done so, we check for type membership
    # in native_types and not in native_numeric_types.
    #
    if obj.__class__ in native_types:
        return True
    try:
        return obj.is_constant()
    except AttributeError:
        pass
    return as_numeric(obj).is_fixed()

def is_fixed(obj):
    """
    A utility function that returns a boolean that indicates whether the
    input object's value is fixed.
    """
    # JDS: NB: I am not sure why we allow str to be a constant, but
    # since we have historically done so, we check for type membership
    # in native_types and not in native_numeric_types.
    #
    if obj.__class__ in native_types:
        return True
    try:
        return obj.is_fixed()
    except AttributeError:
        pass
    return as_numeric(obj).is_fixed()


# It is very common to have only a few constants in a model, but those
# constants get repeated many times.  KnownConstants lets us re-use /
# share constants we have seen before.
KnownConstants = {}

def as_numeric(obj):
    """
    Verify that this obj is a NumericValue or intrinsic value.
    """
    # int and float are *so* common that it pays to treat them specially
    if obj.__class__ in native_numeric_types:
        if obj in KnownConstants:
            return KnownConstants[obj]
        else:
            # Because INT, FLOAT, and sometimes LONG hash the same, we
            # want to convert them to a common type (at the very least,
            # so that the order in which tests run does not change the
            # results!)
            try:
                tmp = float(obj)
                if tmp == obj:
                    tmp = NumericConstant(None, None, tmp)
                    KnownConstants[obj] = tmp
                    return tmp
            except:
                pass

            tmp = NumericConstant(None, None, tmp)
            KnownConstants[obj] = tmp
            return tmp
    try:
        return obj.as_numeric()
    except AttributeError:
        pass
    try: 
        if obj.__class__ is (obj + 0).__class__:
            # obj may (or may not) be hashable, so we need this try
            # block so that things proceed normally for non-hashable
            # "numeric" types
            try:
                if obj in KnownConstants:
                    return KnownConstants[obj]
                else:
                    tmp = NumericConstant(None,None,obj)
                    KnownConstants[obj] = tmp
                    
                    # If we get here, this is a reasonably well-behaving
                    # numeric type: add it to the native numeric types
                    # so that future lookups will be faster.
                    native_numeric_types.add(obj.__class__)
                    # native numeric types are also native types
                    native_types.add(obj.__class__)

                    return tmp
            except:
                return NumericConstant(None, None, obj)
    except:
        pass
    raise ValueError("This object is not a numeric value: %s" % (obj,))


class NumericValue(object):
    """
    This is the base class for numeric values used in Pyomo.

    For efficiency purposes, some derived classes do not call this
    constructor (e.g. see the "Expression" class defined in "expr.py").
    This is fine if the value is finite or unspecified.  In that
    case, no validation is performed, so the subclass can simply
    specify the name, domain and value.

    Constructor Arguments:
        domain          The value domain.  Unspecified default: None.
        name            The value name.  Unspecified default: None
        value           The initial value.  Unspecified default: None.

    Public Class Attributes:
        domain          A domain object that restricts possible values for
                        this object.
        name            A name for this object.
        value           The numeric value of this object.
    """

    __slots__ = ['name','domain','value']

    __hash__ = None

    def __init__(self, name, domain, value):
        self.name = name
        self.domain = domain
        if value is None:
            self.value = None
        elif pyutilib.math.is_nan(value):
            self.value = pyutilib.math.nan
        else:
            self.value = value

    def __getstate__(self):
        """
        This method is required because this class uses slots.

        Derived classes need to over-ride this method to both call the
        base class method (which will populate the dictionary with slots
        specific to the base class, and to add any slots to the
        dictionary unique to the derived class.
        """
        return dict((i, getattr(self, i, None)) for i in NumericValue.__slots__)

    def __setstate__(self, state):
        """
        This method is required because this class uses slots.

        Derived classes need to over-ride this method to both call the
        base class method (which will populate the dictionary with slots
        specific to the base class, and to add any slots to the
        dictionary unique to the derived class.
        """
        for (slot_name, value) in state.iteritems():
            setattr(self, slot_name, value)

    def is_constant(self):
        """Return True if this numeric value is a constant value."""
        return True

    def is_expression(self):
        """Return True if this numeric value is an expression."""
        return False

    def is_relational(self):
        """
        Return True if this numeric value represents a relational
        expression.
        """
        return False

    def is_indexed(self):
        """Return True if this numeric value is an indexed object."""
        return False

    def polynomial_degree(self):
        """Return the polynomial degree of this expression."""
        return 0

    def is_fixed(self):               #pragma:nocover
        """Return True is this is a non-constant value that has been fixed."""
        return False

    def as_numeric(self):
        return self

    def pprint(self, ostream=None, verbose=False):
        """Print the value of this numeric object"""
        raise IOError("NumericValue:pprint is not defined")     #pragma:nocover

    def display(self, ostream=None):    #pragma:nocover
        """Provide a verbose display of this numeric object"""
        self.pprint(ostream=ostream)

    def reset(self):            #pragma:nocover
        """Reset the value of this numeric object"""
        pass                    #pragma:nocover

    def set_value(self, val):
        """Set the value of this numeric object, after validating its value."""
        if self._valid_value(val):
            self.value=val

    def __nonzero__(self):
        """Return True if the value is defined and non-zero."""
        if self.value is None:
            raise ValueError("Numeric value is undefined")
        if self.value:
            return True
        return False

    def __call__(self, exception=True):
        """Return the value of this object."""
        return self.value

    def __float__(self):
        """Coerce the value to a floating point."""
        tmp = self.__call__()
        if tmp is None:
            raise ValueError("Cannot coerce numeric value `%s' to float "
                             "because it is uninitialized." % (self.name,))
        return float(tmp)

    def __int__(self):
        """Coerce the value to an integer point."""
        tmp = self.__call__()
        if tmp is None:
            raise ValueError("Cannot coerce numeric value `%s' to integer "
                             "because it is uninitialized." % (self.name,))
        return int(tmp)

    def _valid_value(self, value, use_exception=True):
        """
        Validate the value.  If use_exception is True, then raise an
        exception.
        """
        ans = value is None or self.domain is None or value in self.domain
        if not ans and use_exception:
            raise ValueError("Numeric value `%s` is not in domain %s"
                             % (value, self.domain))
        return ans

    def __lt__(self,other):
        """Less than operator

        (Called in response to 'self < other' or 'other > self'.)
        """
        return generate_relational_expression('<', self, as_numeric(other))

    def __gt__(self,other):
        """Greater than operator

        (Called in response to 'self > other' or 'other < self'.)
        """
        return generate_relational_expression('<', as_numeric(other), self)

    def __le__(self,other):
        """Less than or equal operator

        (Called in response to 'self <= other' or 'other >= self'.)
        """
        return generate_relational_expression('<=', self, as_numeric(other))

    def __ge__(self,other):
        """Greater than or equal operator

        (Called in response to 'self >= other' or 'other <= self'.)
        """
        return generate_relational_expression('<=', as_numeric(other), self)

    def __eq__(self,other):
        """Equal to operator

        (Called in response to 'self = other'.)
        """
        return generate_relational_expression('==', self, as_numeric(other))

    def __add__(self,other):
        """Binary addition

        (Called in response to 'self + other'.)
        """
        return generate_expression(_add,self,other)

    def __sub__(self,other):
        """ Binary subtraction

        (Called in response to 'self - other'.)
        """
        return generate_expression(_sub,self,other)

    def __mul__(self,other):
        """ Binary multiplication

        (Called in response to 'self * other'.)
        """
        return generate_expression(_mul,self,other)

    def __div__(self,other):
        """ Binary division

        (Called in response to 'self / other'.)
        """
        return generate_expression(_div,self,other)

    def __pow__(self,other):
        """ Binary power

        (Called in response to 'self ** other'.)
        """
        return generate_expression(_pow,self,other)

    def __radd__(self,other):
        """Binary addition

        (Called in response to 'other + self'.)
        """
        return generate_expression(_radd,self,other)

    def __rsub__(self,other):
        """ Binary subtraction

        (Called in response to 'other - self'.)
        """
        return generate_expression(_rsub,self,other)

    def __rmul__(self,other):
        """ Binary multiplication

        (Called in response to 'other * self'.)
        """
        return generate_expression(_rmul,self,other)

    def __rdiv__(self,other):
        """ Binary division

        (Called in response to 'other / self'.)
        """
        return generate_expression(_rdiv,self,other)

    def __rpow__(self,other):
        """ Binary power

        (Called in response to 'other ** self'.)
        """
        return generate_expression(_rpow,self,other)

    def __iadd__(self,other):
        """Binary addition

        (Called in response to 'self += other'.)
        """
        return generate_expression(_iadd,self,other)

    def __isub__(self,other):
        """ Binary subtraction

        (Called in response to 'self -= other'.)
        """
        return generate_expression(_isub,self,other)

    def __imul__(self,other):
        """ Binary multiplication

        (Called in response to 'self *= other'.)
        """
        return generate_expression(_imul,self,other)

    def __idiv__(self,other):
        """ Binary division

        (Called in response to 'self /= other'.)
        """
        return generate_expression(_idiv,self,other)

    def __ipow__(self,other):
        """ Binary power

        (Called in response to 'self **= other'.)
        """
        return generate_expression(_ipow,self,other)

    def __neg__(self):
        """ Negation

        (Called in response to '- self'.)
        """
        return generate_expression(_neg,self, None)

    def __pos__(self):
        """ Positive expression

        (Called in response to '+ self'.)
        """
        return self

    def __abs__(self):
        """ Absolute value

        (Called in response to 'abs(self)'.)
        """
        return generate_expression(_abs,self, None)


class NumericConstant(NumericValue):
    """An object that contains a constant numeric value.

    If domain is None, then the domain is assumed to be Reals.
    Additionally, this class does not validate the values of numeric
    constants.  The user specifies the value, so we assume that the
    value is valid.

    Constructor Arguments:
        name            The value name.  Unspecified default: None
        domain          The value domain.  Unspecified default: None.
        value           The initial value.  Unspecified default: None.
    """


    __slots__ = [] # adds no attributes above and beyond those defined in the base class.

    def __init__(self, name, domain, value):
        if domain is None:
            self.domain = Reals
        else:
            self.domain = domain
        self.name = name
        if pyutilib.math.is_nan(value):
            self.value = pyutilib.math.nan
        else:
            self.value = value

    def is_fixed(self):
        return True

    def __str__(self):
        return str(self.value)

    def pprint(self, ostream=None, verbose=False):
        if ostream is None:         #pragma:nocover
            ostream = sys.stdout
        print >>ostream, str(self),

# We use as_numeric() so that the constant is also in the cache
ZeroConstant = as_numeric(0)

# TODO - Why is this here???
from expr import generate_expression, generate_relational_expression, \
     _add, _sub, _mul, _div, _pow, _neg, _abs, _inplace, \
     _radd, _rsub, _rmul, _rdiv, _rpow, _iadd, _isub, _imul, _idiv, _ipow



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

from sys import stderr as SE

import pyutilib.component.core


pyutilib.component.core.PluginGlobals.push_env( 'coopr.pyomo' )

from base import *
import base.pyomo as pyomo
from expr import *
import data
from io import *
from components import *
import preprocess
import coopr.opt
import transform
import scripting
import check

pyutilib.component.core.PluginGlobals.pop_env()

try:
    import pkg_resources
    #
    # Load modules associated with Plugins that are defined in
    # EGG files.
    #
    for entrypoint in pkg_resources.iter_entry_points('coopr.pyomo'):
        try:
            plugin_class = entrypoint.load()
        except Exception, err:
            SE.write( "Error loading coopr.pyomo entry point: %s  entrypoint='%s'\n" % (err, entrypoint) )
except Exception, err:
    SE.write( "Error loading coopr.pyomo entry points\n" )




#
# Test that the Pyomo NL writer does not modify the original model
#

import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__))+os.sep
import re
import glob

import pyutilib.th as unittest
import json
from coopr.pyomo import *
from coopr.pyomo.base.var import _VarData
from coopr.opt import ProblemFormat
import StringIO
from coopr.pyomo.base import expr as Expr
import itertools

# NOTE: The following line and the call to reload() in the function 'nlwriter_exprmod_test' are absolutely
#       critical so that the test suite that checks if expressions are modified by the
#       NL writer will not erroneously report as all passing. The test functions seems to 
#       pre-execute when run through 'test.coopr' and therefore if the model is modified it will 
#       actually start out that way in the true test, and therefore pass.
sys.path.append(currdir)

"""
This following function checks that expr1 is an
identical clone of expr2. By this I mean, all nested expressions
are the same, and all contained variables reference the same object
in memory ((Var1 is Var2) is True).
"""
@unittest.nottest
def expressions_equal(expr1,expr2):
    if not type(expr1) == type(expr2):
        print type(expr1), "!=", type(expr2)
        return False
    if isinstance(expr1,Var) or isinstance(expr1,_VarData):
        if not expr1 is expr2:
            print expr1, "is not", expr2
            return False
    elif isinstance(expr1,Expr.Expression):
        if not expr1.is_constant() is expr2.is_constant():
            print expr1.is_constant(), "is not", expr2.is_constant()
            return False
        if not expr1.polynomial_degree() == expr2.polynomial_degree():
            print expr1.polynomial_degree(), "!=", expr2.polynomial_degree()
            return False
        if isinstance(expr1,Expr._ProductExpression):
            if not expr1.coef == expr2.coef:
                print expr1.coef, "!=", expr2.coef
                return False
            if not len(expr1._denominator) == len(expr2._denominator):
                print len(expr1._denominator), "!=", len(expr2._denominator)
                return False
            if not all(expressions_equal(expr1._denominator[i],expr2._denominator[i]) for i in xrange(len(expr1._denominator))):
                return False
            if not len(expr1._numerator) == len(expr2._numerator):
                print len(expr1._numerator), "!=", len(expr2._numerator)
                return False
            if not all(expressions_equal(expr1._numerator[i],expr2._numerator[i]) for i in xrange(len(expr1._numerator))):
                return False
        if isinstance(expr1,Expr._SumExpression):
            if not expr1._const == expr2._const:
                print expr1._const, "!=", expr2._const
                return False
            if not len(expr1._coef) == len(expr2._coef):
                print len(expr1._coef), "!=", len(expr2._coef)
                return False
            if not all(expressions_equal(expr1._coef[i],expr2._coef[i]) for i in xrange(len(expr1._coef))):
                return False
            if not all(expressions_equal(expr1._args[i],expr2._args[i]) for i in xrange(len(expr1._args))):
                return False
        if isinstance(expr1,Expr._IdentityExpression) or \
           isinstance(expr1,Expr._EqualityExpression):
            if not len(expr1._args) == len(expr2._args):
                print len(expr1._args), "!=", len(expr2._args)
                return False
            if not all(expressions_equal(expr1._args[i],expr2._args[i]) for i in xrange(len(expr1._args))):
                return False
        if isinstance(expr1,Expr._InequalityExpression):
            if not len(expr1._args) == len(expr2._args):
                print len(expr1._args), "!=", len(expr2._args)
                return False
            if not all(expressions_equal(expr1._args[i],expr2._args[i]) for i in xrange(len(expr1._args))):
                return False
            if not len(expr1._cloned_from) == len(expr2._cloned_from):
                print len(expr1._cloned_from), "!=", len(expr2._cloned_from)
                return False
            if not all(expressions_equal(expr1._cloned_from[i],expr2._cloned_from[i]) for i in xrange(len(expr1._cloned_from))):
                return False
            
        # check that they pprint the same
        output1 = StringIO.StringIO()
        output2 = StringIO.StringIO()
        try:
            expr1.pprint(ostream=output1)
        except:
            output1.write(str(repr(expr1)))
        try:
            expr2.pprint(ostream=output2)
        except:
            output2.write(str(repr(expr2)))
        if not output1.getvalue() == output2.getvalue():
            print output1.getvalue(), "!=", output2.getvalue()
            output1.close()
            output2.close()
            return False
        output1.close()
        output2.close()
    else:
        if not value(expr1) == value(expr2):
            print value(expr1), "!=", value(expr2)
            return False

    return True

class ExprModTests(unittest.TestCase): pass
ExprModTests = unittest.category('smoke', 'nightly','expensive')(ExprModTests)


"""
The following test checks that sending a model to the nl writer does not
modify the original expressions on the model.
"""
@unittest.nottest
def nlwriter_exprmod_test(self,name):
    
    m = __import__(name)
    # This is critical, see note above
    reload(m)
    model = None
    if os.path.exists(currdir+name+'.dat'):
        model = m.model.create(filename=currdir+name+'.dat')
    else:
        model = m.model.create()
    del m
    model.concrete_mode()

    # Clone the original expressions
    Obj = model.active_components(Objective)
    orig_model_exprs = [ Obj[obj][i].expr.clone() for obj in Obj for i in Obj[obj] if isinstance(Obj[obj][i].expr,Expr.Expression) ]
    orig_model_exprs.extend([ Obj[obj][i].expr for obj in Obj for i in Obj[obj] if not isinstance(Obj[obj][i].expr,Expr.Expression) ])
    orig_model_exprs.extend([constraint[index].body.clone() \
                              for block in model.all_blocks() \
                              for constraint in itertools.chain(block.active_components(Constraint).itervalues(), block.active_components(ConstraintList).itervalues()) \
                              for index in sorted(constraint.keys()) if isinstance(constraint[index].body,Expr.Expression)])
    # Don't try to clone things that can't be cloned, e.g., a single var    
    orig_model_exprs.extend([constraint[index].body \
                              for block in model.all_blocks() \
                              for constraint in itertools.chain(block.active_components(Constraint).itervalues(), block.active_components(ConstraintList).itervalues()) \
                              for index in sorted(constraint.keys()) if not isinstance(constraint[index].body,Expr.Expression)])
    
    # Send the model to the nl writer
    model.write(filename=currdir+'exprtest.nl', format=ProblemFormat.nl, generate_symbolic_labels=False)

    # Generate a list of new model expressions
    Obj = model.active_components(Objective)
    model_exprs = [ Obj[obj][i].expr for obj in Obj for i in Obj[obj] if isinstance(Obj[obj][i].expr,Expr.Expression) ]
    model_exprs.extend([ Obj[obj][i].expr for obj in Obj for i in Obj[obj] if not isinstance(Obj[obj][i].expr,Expr.Expression) ])
    model_exprs.extend([constraint[index].body \
                              for block in model.all_blocks() \
                              for constraint in itertools.chain(block.active_components(Constraint).itervalues(), block.active_components(ConstraintList).itervalues()) \
                              for index in sorted(constraint.keys()) if isinstance(constraint[index].body,Expr.Expression)])
    # Don't try to clone things that can't be cloned, e.g., a single var    
    model_exprs.extend([constraint[index].body \
                              for block in model.all_blocks() \
                              for constraint in itertools.chain(block.active_components(Constraint).itervalues(), block.active_components(ConstraintList).itervalues()) \
                              for index in sorted(constraint.keys()) if not isinstance(constraint[index].body,Expr.Expression)])
    
    # Check that the expression in the new list match those in the old list
    for (expr1,expr2) in itertools.izip(model_exprs, orig_model_exprs):
        if not expressions_equal(expr1,expr2):
            try:
                expr1.pprint()
            except:
                print str(repr(expr1))
            try:
                expr2.pprint()
            except:
                print str(repr(expr2))
            self.fail(msg="expression not equal")
                
    # delete temp test files
    os.remove(currdir+'exprtest.nl')
    

for f in glob.glob(currdir+'*_testCase.py'):
    name = re.split('.',os.path.basename(f))[0]
    name = os.path.basename(f).replace('.py','')
    ExprModTests.add_fn_test(fn=nlwriter_exprmod_test, name=name)

if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

#
# Utility functions
#

__all__ = ['summation', 'dot_product', 'sequence', 'xsequence']

import var
import expr

def summation(*args, **kwds):
    """
    A utility function to compute a generalized dot product.  The following examples illustrate
    the use of this function:

    summation(x)
    Sum the elements of x

    summation(x,y)
    Sum the product of elements in x and y

    summation(x,y, index=z)
    Sum the product of elements in x and y, over the index set z

    summation(x, denom=a)
    Sum the product of x_i/a_i

    summation(denom=(a,b))
    Sum the product of 1/(a_i*b_i)
    """

    denom = kwds.pop('denom', tuple() )

    if type(denom) not in (list, tuple):
        denom = [denom]
    if len(args) == 0 and len(denom) == 0:
        raise ValueError, "The summation() command requires at least an " + \
              "argument or a denominator term"
    if 'index' in kwds:
        index=kwds['index']
    else:
        if len(args) > 0:
            iarg=args[-1]
            if not isinstance(iarg,var.Var):
                raise ValueError, "Error executing summation(): The last argument value must be a variable object if no 'index' option is specified"
        else:
            iarg=denom[-1]
            if not isinstance(iarg,var.Var):
                raise ValueError, "Error executing summation(): The last denom argument value must be a variable object if no 'index' option is specified"
        index = iarg.keys()

    ans = 0
    num_index = range(0,len(args))
    denom_index = range(0,len(denom))
    #
    # Iterate through all indices
    #
    for i in index:
        #
        # Iterate through all arguments
        #
        item = 1
        for j in num_index:
            item *= args[j][i]
        for j in denom_index:
            item /= denom[j][i]
        ans += item
    return ans

def dot_product(*args, **kwds):
    """
    A synonym for the summation() function.
    """
    return summation(*args, **kwds)

# This is deprecated.  The same functionality is provided by
# itertools.product
def multisum(*args):
    """
    Returns a generator that produces all possible permutations from an
    arbitrary set of containers.

    Example:

    >>> a = [1,2,3]
    >>> b = ['foo','bar']
    >>> c = [4,5,6]

    >>> for (i,j,k) in multisum(a,b,c):
            print i,j,k

    1 foo 4
    2 foo 4
    3 foo 4
    1 bar 4
    2 bar 4
    3 bar 4
    1 foo 5
    2 foo 5
    3 foo 5
    1 bar 5
    2 bar 5
    3 bar 5
    1 foo 6
    2 foo 6
    3 foo 6
    1 bar 6
    2 bar 6
    3 bar 6

    """
    nargs = len(args)

    # Generators
    gens = [arg.__iter__() for arg in args]

    # Current value list; note that this will fail if any container is empty,
    # which is the desired behavior
    current = [g.next() for g in gens]

    i = 0
    while i < nargs:
        yield current

        # Find the next legal iterator, or break if we run out
        while True:
            if i >= nargs:
                break
            try:
                current[i] = gens[i].next()
                i = 0
                break
            except StopIteration:
                gens[i] = args[i].__iter__()
                current[i] = gens[i].next()
                i += 1


def sequence(*args):
    """
    sequence([start,] stop[, step]) -> list of integers

    Return a list containing an arithmetic progression of integers.
    sequence(i, j) returns [i, i+1, i+2, ..., j]; start (!) defaults to 1.
    When step is given, it specifies the increment (or decrement).
    For example, sequence(4) returns [1, 2, 3, 4].
    """
    if len(args) == 0:
        raise TypeError, 'sequence expected at least 1 arguments, got 0'
    if len(args) > 3:
        raise TypeError, 'sequence expected at most 3 arguments, got %d' % len(args)
    if len(args) == 1:
        return range(1,args[0]+1)
    if len(args) == 2:
        return range(args[0],args[1]+1)
    return range(args[0],args[1]+1,args[2])

def xsequence(*args):
    """
    xsequence([start,] stop[, step]) -> xrange object

    Like sequence(), but instead of returning a list, returns an object that
    generates the numbers in the sequence on demand.  For looping, this is
    slightly faster than sequence() and more memory efficient.
    """
    if len(args) == 0:
        raise TypeError, 'xsequence expected at least 1 arguments, got 0'
    if len(args) > 3:
        raise TypeError, 'xsequence expected at most 3 arguments, got %d' % len(args)
    if len(args) == 1:
        return xrange(1,args[0]+1)
    if len(args) == 2:
        return xrange(args[0],args[1]+1)
    return xrange(args[0],args[1]+1,args[2])

def isfunctor(obj):
    """
    Returns true iff obj.__call__ is defined.
    """
    try:
        obj.__call__
        return True
    except:
        return False



#
# Unit Tests for complete examples
#

import os
import sys
from os.path import abspath, dirname
topdir = dirname(dirname(abspath(__file__)))+os.sep+".."+os.sep+".."
sys.path.insert(0, topdir)
currdir = dirname(abspath(__file__))+os.sep

from coopr.pyomo import *
import pyutilib.th as unittest
import glob

class TestAmplbook2(unittest.TestCase): pass

#
# DISABLED FOR NOW ... Revisit these when the ipconvert utility is stable...
#

#data_dir=topdir+os.sep+"examples"+os.sep+"pyomo"+os.sep+"amplbook2"+os.sep
#files = glob.glob(data_dir+"*.py")
#for file in files:
#    bname=os.path.basename(file)
#    name=bname.split('.')[0]
#    TestAmplbook2.add_commandline_test(cmd="cd "+data_dir+"; "+topdir+os.sep+"scripts/pyomo "+bname+" "+name+".dat", baseline=data_dir+name+".log", name=name)

if __name__ == "__main__":
    unittest.main()




from coopr.pyomo import *
import math

N = 5
M = 6
P = 3

model = ConcreteModel()

model.Locations = RangeSet(1,N)

model.Customers = RangeSet(1,M)

def d_rule(model, n, m):
    return math.sin(n*2.33333+m*7.99999)
model.d = Param(model.Locations, model.Customers, rule=d_rule, within=Reals)

model.x = Var(model.Locations, model.Customers, bounds=(0.0,1.0))

model.y = Var(model.Locations, within=Binary)

def rule(model):
    return sum( [model.d[n,m]*model.x[n,m] for n in model.Locations for m in model.Customers] )
model.obj = Objective(rule=rule)

model.single_x = ConstraintList()
for m in model.Customers:
    model.single_x.add( sum( [model.x[n,m] for n in model.Locations]) == 1.0 )

model.bound_y = ConstraintList()
for n in model.Locations:
    for m in model.Customers:
        model.bound_y.add( model.x[n,m] <= model.y[n] )

model.num_facilities = Constraint(expr=sum( [model.y[n] for n in model.Locations] ) == P)



import pyutilib.th as unittest
from coopr.opt import *
from model import model
import coopr
from coopr.pyomo.base import Var

try:
    cplex = coopr.plugins.solvers.CPLEX(keepFiles=True)
    cplex_available = (not cplex.executable() is None) and cplex.available(False)
    asl = coopr.plugins.solvers.ASL(keepFiles=True, options={'solver':'cplexamp'})
    cplexamp_available = cplex_available and (not asl.executable() is None) and asl.available(False)
    try:
        import cplex
        cplexpy_available = True
    except ImportError:
        cplexpy_available = False
except pyutilib.common.ApplicationError:
    cplexamp_available=False

try:
    gurobi = coopr.plugins.solvers.GUROBI(keepFiles=True)
    gurobi_available = (not gurobi.executable() is None) and gurobi.available(False)
    asl = coopr.plugins.solvers.ASL(keepFiles=True, options={'solver':'gurobi_ampl'})
    gurobi_ampl_available = gurobi_available and (not asl.executable() is None) and asl.available(False)
    try:
        import gurobipy
        gurobi_py_available = True
        gurobi_py_version = gurobipy.gurobi.version()
    except ImportError:
        gurobi_py_available = False
        gurobi_py_version = None
except pyutilib.common.ApplicationError:
    gurobi_ampl_available=False

diff_tol = 1e-4

class WriterTests(unittest.TestCase):

    @unittest.skipIf(not (cplex_available and cplexamp_available),"The 'cplex' executable is not available")
    def testCPLEXnl(self):
        self.instancelp=model.create()
        self.instancenl=model.create()

        self.optlp=SolverFactory("cplex",solver_io="lp")
        self.optnl=SolverFactory("cplex",solver_io="nl")

        self.resultslp=self.optlp.solve(self.instancelp)
        self.resultsnl=self.optnl.solve(self.instancenl)

        self.instancelp.load(self.resultslp)
        self.instancenl.load(self.resultsnl)
        
        self.assertAlmostEqual(self.instancelp.obj(),self.instancenl.obj(), delta=diff_tol)
        for i, val in self.instancelp.active_components(Var).iteritems():
            self.assertAlmostEqual( val(),
                              self.instancenl.active_components(Var)[i](), delta=diff_tol)

    @unittest.skipIf(not (cplex_available and cplexpy_available),"The 'cplex' python interface is not available")
    def testCPLEXpy(self):
        self.instancelp=model.create()
        self.instancepy=model.create()

        self.optlp=SolverFactory("cplex",solver_io="lp")
        self.optpy=SolverFactory("cplex",solver_io="python")

        self.resultslp=self.optlp.solve(self.instancelp)
        self.resultspy=self.optpy.solve(self.instancepy)

        self.instancelp.load(self.resultslp)
        self.instancepy.load(self.resultspy)
        
        self.assertAlmostEqual(self.instancelp.obj(),self.instancepy.obj(), delta=diff_tol)
        for i, val in self.instancelp.active_components(Var).iteritems():
            self.assertAlmostEqual( val(),
                              self.instancepy.active_components(Var)[i](), delta=diff_tol)

    @unittest.skipIf(not (gurobi_available and gurobi_ampl_available),"The 'gurobi executable is not available") 
    def testGUROBInl(self):
        self.instancelp=model.create()
        self.instancenl=model.create()

        self.optlp=SolverFactory("gurobi",solver_io="lp")
        self.optnl=SolverFactory("gurobi_ampl",solver_io="nl")

        self.resultslp=self.optlp.solve(self.instancelp)
        self.resultsnl=self.optnl.solve(self.instancenl)

        self.instancelp.load(self.resultslp)
        self.instancenl.load(self.resultsnl)
        
        self.assertAlmostEqual(self.instancelp.obj(),self.instancenl.obj(), delta=diff_tol)
        for i, val in self.instancelp.active_components(Var).iteritems():
            self.assertAlmostEqual( val(),
                              self.instancenl.active_components(Var)[i](), delta=diff_tol)

    # these models have quadratic constraints and objectives - only gurobi major version >= 5 can handle them.
    @unittest.skipIf((not (gurobi_available and gurobi_py_available)) or (gurobi_py_available and gurobi_py_version[0] <= 4),"The 'gurobi python interface is not available or the major version is < 5") 
    def testGUROBIpy(self):
        self.instancelp=model.create()
        self.instancepy=model.create()

        self.optlp=SolverFactory("gurobi",solver_io="lp")
        self.optpy=SolverFactory("gurobi",solver_io="python")

        self.resultslp=self.optlp.solve(self.instancelp)
        self.resultspy=self.optpy.solve(self.instancepy)

        self.instancelp.load(self.resultslp)
        self.instancepy.load(self.resultspy)
        
        self.assertAlmostEqual(self.instancelp.obj(),self.instancepy.obj(), delta=diff_tol)
        for i, val in self.instancelp.active_components(Var).iteritems():
            self.assertAlmostEqual( val(),
                              self.instancepy.active_components(Var)[i](), delta=diff_tol)


if __name__ == "__main__":
    unittest.main()



## variant of diet.py that presents constraints and objectives twice,
## to see how terms are collected

#
# Imports
#
from coopr.pyomo import *

#
# Setup
#

model = AbstractModel()
model.NUTR = Set()
model.FOOD = Set()

model.cost = Param(model.FOOD, within=NonNegativeReals)

model.f_min = Param(model.FOOD, within=NonNegativeReals)

def f_max_valid (model, value, j):
    return model.f_max[j] > model.f_min[j]
model.f_max = Param(model.FOOD, validate=f_max_valid)

model.n_min = Param(model.NUTR, within=NonNegativeReals)

def paramn_max (model, i):
    model.n_max[i] > model.n_min[i]
    return model.n_max[i]
model.n_max = Param(model.NUTR, initialize=paramn_max)

# ***********************************

model.amt = Param(model.NUTR, model.FOOD, within=NonNegativeReals)

def Buy_bounds(model, i):
    return (model.f_min[i],model.f_max[i])
model.Buy = Var(model.FOOD, bounds=Buy_bounds)

def Objective_rule(model):
    ans = 0
    for j in model.FOOD:
        ans = ans + model.cost[j] * model.Buy[j]
    for j in model.FOOD:
        ans = ans + model.cost[j] * model.Buy[j]
    return ans
model.totalcost = Objective(rule=Objective_rule)

def Diet_rule(model, i):
    expr = 0
    for j in model.FOOD:
        expr = expr + model.amt[i,j] * model.Buy[j]
    for j in model.FOOD:
        expr = expr + model.amt[i,j] * model.Buy[j]
    expr = expr > 2*model.n_min[i]
    expr = expr < 2*model.n_max[i]
    return expr
model.Diet = Constraint(model.NUTR, rule=Diet_rule)



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

import os.path
from TableData import TableData
from pyutilib.excel import ExcelSpreadsheet
from pyutilib.component.core import alias
import pyutilib.common


class SheetTable(TableData):

    alias("xls", "Manage IO with Excel XLS files.")

    def __init__(self):
        TableData.__init__(self)

    def open(self):
        if self.filename is None:
            raise IOError, "No filename specified"
        if not os.path.exists(self.filename):
            raise IOError, "Cannot find file '%s'" % self.filename
        self.sheet = None
        if self._data is not None:
            self.sheet = self._data
        else:
            try:
                self.sheet = ExcelSpreadsheet(self.filename)
            except pyutilib.common.ApplicationError:
                raise

    def read(self):
        if self.sheet is None:
            return
        tmp = self.sheet.get_range(self.options.range, raw=True)
        if type(tmp) in (int,long,float):
            if not self.options.param is None:
                self._info = ["param",self.options.param,":=",tmp]
            elif len(self.options.symbol_map) == 1:
                self._info = ["param",self.options.symbol_map[self.options.symbol_map.keys()[0]],":=",tmp]
            else:
                raise IOError, "Data looks like a parameter, but multiple parameter names have been specified: %s" % str(self.options.symbol_map)
        elif len(tmp) == 0:
            raise IOError, "Empty range '%s'" % self.options.range
        else:
	    tmp = [list(x) for x in tmp]
            self._set_data(tmp[0], tmp[1:])
        return True

    def close(self):
        if self._data is None and not self.sheet is None:
            del self.sheet
